/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * The implementation of the VPInterpreter class.
 */


#include <gtk/gtkwidget.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkmarshal.h>

#include "pylib.h"
#include "gvpvalue.h"

#include "gvpinterpreter.h"


GtkObject *vp_interpreter_main = NULL;


/* The signals. */

enum {
	SCRIPT_FAILED,
	INTERNAL_ERROR,
	NR_SIGNALS
};

static guint interpreter_signals[NR_SIGNALS];

static GtkObjectClass *parent_class;


static void vp_interpreter_class_init(VPInterpreterClass *);
static void vp_interpreter_init(VPInterpreter *);
static void vp_interpreter_destroy(GtkObject *);
static void vp_error_handler(const char *,void *);


/*
 * Return the type ID.
 */
 
guint vp_interpreter_get_type(void)
{
        static guint interpreter_type = 0;
 
        if (interpreter_type == 0)
        {
                static const GtkTypeInfo interpreter_info = {
                        "VPInterpreter",
                        sizeof (VPInterpreter),
                        sizeof (VPInterpreterClass),
                        (GtkClassInitFunc)vp_interpreter_class_init,
                        (GtkObjectInitFunc)vp_interpreter_init,
                };
 
                interpreter_type = gtk_type_unique(gtk_object_get_type(),&interpreter_info);
	}
 
        return interpreter_type;
}


/*
 * Initialise the class structure.
 */
 
static void vp_interpreter_class_init(VPInterpreterClass *ic)
{
	GtkObjectClass *oc = (GtkObjectClass *)ic;

	parent_class = gtk_type_class(gtk_object_get_type());

	/* Add the signals. */

	interpreter_signals[SCRIPT_FAILED] = gtk_signal_new(
		"script_failed",
		GTK_RUN_FIRST,
		oc -> type,
		GTK_SIGNAL_OFFSET(VPInterpreterClass,script_failed),
		gtk_marshal_NONE__POINTER_POINTER,
		GTK_TYPE_NONE,
		2,
		GTK_TYPE_STRING,
		GTK_TYPE_STRING);

	ic -> script_failed = NULL;

	interpreter_signals[INTERNAL_ERROR] = gtk_signal_new(
		"internal_error",
		GTK_RUN_FIRST,
		oc -> type,
		GTK_SIGNAL_OFFSET(VPInterpreterClass,internal_error),
		gtk_marshal_NONE__POINTER,
		GTK_TYPE_NONE,
		1,
		GTK_TYPE_STRING);

	ic -> internal_error = NULL;

	gtk_object_class_add_signals(oc,interpreter_signals,NR_SIGNALS);

	/* Set the dtor. */

	oc -> destroy = vp_interpreter_destroy;
}
 
 
/*
 * Initialise the instance structure.
 */
 
static void vp_interpreter_init(VPInterpreter *this)
{
	this -> pre1code = "import vp, libgvp";
	this -> pre2code = NULL;
	this -> postcode = NULL;
	this -> context = pylibInit(vp_error_handler,this);
}


/*
 * Create an interpreter.  defAppWindow is the default application window to
 * make available to all scripts run in this interpreter.
 */

GtkObject *vp_interpreter_new(GtkWidget *defAppWindow)
{
	GtkObject *o;
	VPInterpreter *this;

	this = gtk_type_new(vp_interpreter_get_type());

	o = GTK_OBJECT(this);

	vp_interpreter_add_application_window(o,defAppWindow,NULL);

	if (vp_interpreter_main == NULL)
		vp_interpreter_main = o;

	return o;
}


/*
 * The dtor.
 */

static void vp_interpreter_destroy(GtkObject *o)
{
	VPInterpreter *this;

	g_return_if_fail(o != NULL);
	g_return_if_fail(VP_IS_INTERPRETER(o));

	this = VP_INTERPRETER(o);

	pylibDone(this -> context,this -> postcode);

	this -> context = NULL;

	if (vp_interpreter_main == o)
		vp_interpreter_main = NULL;

	/* Chain up. */

	if (parent_class -> destroy)
		(*parent_class -> destroy)(o);
}


/*
 * Set the VP initialisation code.
 */

void vp_interpreter_set_vp_init_code(GtkObject *o,char *code)
{
	VPInterpreter *this;

	g_return_if_fail(o != NULL);
	g_return_if_fail(VP_IS_INTERPRETER(o));

	this = VP_INTERPRETER(o);

	this -> pre1code = code;
}


/*
 * Set the application initialisation code.
 */

void vp_interpreter_set_app_init_code(GtkObject *o,char *code)
{
	VPInterpreter *this;

	g_return_if_fail(o != NULL);
	g_return_if_fail(VP_IS_INTERPRETER(o));

	this = VP_INTERPRETER(o);

	this -> pre2code = code;
}


/*
 * Execute a script in the interpreter.  script is the name of a file
 * containing the script.
 */

void vp_interpreter_execute(GtkObject *o,char *script)
{
	const char *emess;
	VPInterpreter *this;

	g_return_if_fail(o != NULL);
	g_return_if_fail(VP_IS_INTERPRETER(o));

	this = VP_INTERPRETER(o);

	emess = pylibExec(this -> context,script,this -> pre1code,this -> pre2code);

	if (emess != NULL)
		gtk_signal_emit(o,interpreter_signals[SCRIPT_FAILED],script,emess);
}


/*
 * Set the application done code.
 */

void vp_interpreter_set_app_done_code(GtkObject *o,char *code)
{
	VPInterpreter *this;

	g_return_if_fail(o != NULL);
	g_return_if_fail(VP_IS_INTERPRETER(o));

	this = VP_INTERPRETER(o);

	this -> postcode = code;
}


/*
 * Tell the Python library about an application window.  name is the name a
 * script will use to refer to the window or NULL if this is the default
 * window.  appWindow is the application window.
 */

void vp_interpreter_add_application_window(GtkObject *o,GtkWidget *appWindow,const char *name)
{
	VPInterpreter *this;

	g_return_if_fail(o != NULL);
	g_return_if_fail(VP_IS_INTERPRETER(o));

	this = VP_INTERPRETER(o);

	pylibAddAppWindow(this -> context,appWindow,name);
}


/*
 * Tell the Python library that an application window has been removed.  name
 * is the name a script will use to refer to the window.  The default window
 * cannot be removed, only changed.
 */

void vp_interpreter_remove_application_window(GtkObject *o,GtkWidget *appWindow)
{
	VPInterpreter *this;

	g_return_if_fail(o != NULL);
	g_return_if_fail(VP_IS_INTERPRETER(o));

	this = VP_INTERPRETER(o);

	pylibRemoveAppWindow(this -> context,appWindow);
}


/*
 * Get the context for an interpreter.
 */

struct pylibContext *vp_interpreter_get_context(GtkObject *o)
{
	VPInterpreter *this;

	if (o == NULL || (o = vp_interpreter_main) == NULL)
		return NULL;

	g_return_val_if_fail(VP_IS_INTERPRETER(o),NULL);

	this = VP_INTERPRETER(o);

	return this -> context;
}


/*
 * Set an application window's argv property.
 */

void vp_interpreter_set_argv(GtkObject *o,int argc,char **argv,GtkWidget *appWindow)
{
	VPInterpreter *this;

	g_return_if_fail(o != NULL);
	g_return_if_fail(VP_IS_INTERPRETER(o));

	this = VP_INTERPRETER(o);

	pylibSetAppWindowArgv(this -> context,appWindow,argc,argv);
}


/*
 * Set the signals for all application windows.
 */

void vp_interpreter_set_signals(GtkObject *o,char **signames)
{
	VPInterpreter *this;

	g_return_if_fail(o != NULL);
	g_return_if_fail(VP_IS_INTERPRETER(o));

	this = VP_INTERPRETER(o);

	pylibSetAppWindowSignals(this -> context,signames);
}


/*
 * Emit an application window signal.
 */

void vp_interpreter_emit_signal(GtkObject *o,char *signame,GtkObject *val,GtkWidget *appWindow)
{
	VPInterpreter *this;
	PyObject *pyval;

	g_return_if_fail(o != NULL);
	g_return_if_fail(VP_IS_INTERPRETER(o));

	this = VP_INTERPRETER(o);

	if (val != NULL)
	{
		VPValue *valvi;

		g_return_if_fail(VP_IS_VALUE(val));

		valvi = VP_VALUE(val);

		pyval = valvi -> pyobj;
	}
	else
		pyval = NULL;

	pylibEmitAppWindowSignal(this -> context,appWindow,signame,pyval);
}


/*
 * This is the error handler called by the Python library whenever there has
 * been a significant internal error.  emess describes the nature of the error.
 */

static void vp_error_handler(const char *emess,void *tkData)
{
	gtk_signal_emit((GtkObject *)tkData,interpreter_signals[INTERNAL_ERROR],emess);
}
