/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * The implementation of the VPValue, VPValueDict and VPValueList types.
 */


#include "pylib.h"
#include "gvpinterpreter.h"

#include "gvpvalue.h"


static GtkObjectClass *parent_class;


/*****************************************************************************/
/* VPValue                                                                   */
/*****************************************************************************/

static void vp_value_class_init(VPValueClass *);
static void vp_value_init(VPValue *);
static void vp_value_destroy(GtkObject *);


/*
 * Return the type ID.
 */

guint vp_value_get_type(void)
{
	static guint value_type = 0;

	if (value_type == 0)
	{
		static const GtkTypeInfo value_info = {
			"VPValue",
			sizeof (VPValue),
			sizeof (VPValueClass),
			(GtkClassInitFunc)vp_value_class_init,
			(GtkObjectInitFunc)vp_value_init,
		};

		value_type = gtk_type_unique(gtk_object_get_type(),&value_info);
	}

	return value_type;
}


/*
 * Initialise the class structure.
 */

static void vp_value_class_init(VPValueClass *vc)
{
	((GtkObjectClass *)vc) -> destroy = vp_value_destroy;

	 parent_class = gtk_type_class(gtk_object_get_type());
}


/*
 * Initialise the instance structure.
 */

static void vp_value_init(VPValue *this)
{
	this -> pyobj = NULL;
	this -> context = NULL;
}


/*
 * Create a string value.
 */

GtkObject *vp_value_new_string(const char *strval,GtkObject *interp)
{
	VPValue *this;
	struct pylibContext *context;

	this = gtk_type_new(vp_value_get_type());

	if ((this -> context = vp_interpreter_get_context(interp)) != NULL)
		this -> pyobj = pylibValueCtorString(this -> context,strval);

	return GTK_OBJECT(this);
}


/*
 * Create a long value.
 */

GtkObject *vp_value_new_long(long longval,GtkObject *interp)
{
	VPValue *this;

	this = gtk_type_new(vp_value_get_type());

	if ((this -> context = vp_interpreter_get_context(interp)) != NULL)
		this -> pyobj = pylibValueCtorLong(this -> context,longval);

	return GTK_OBJECT(this);
}


/*
 * Create a double value.
 */

GtkObject *vp_value_new_double(double doubleval,GtkObject *interp)
{
	VPValue *this;

	this = gtk_type_new(vp_value_get_type());

	if ((this -> context = vp_interpreter_get_context(interp)) != NULL)
		this -> pyobj = pylibValueCtorDouble(this -> context,doubleval);

	return GTK_OBJECT(this);
}


/*
 * The dtor.
 */

static void vp_value_destroy(GtkObject *o)
{
	VPValue *this;

	g_return_if_fail(o != NULL);
	g_return_if_fail(VP_IS_VALUE(o));

	this = VP_VALUE(o);

	if (this -> context != NULL)
		pylibValueDtor(this -> context,this -> pyobj);

	this -> pyobj = NULL;
	this -> context = NULL;

	/* Chain up. */

	if (parent_class -> destroy)
		(*parent_class -> destroy)(o);
}


/*****************************************************************************/
/* VPValueDict                                                               */
/*****************************************************************************/

static void vp_valuedict_class_init(VPValueDictClass *);
static void vp_valuedict_init(VPValueDict *);


/*
 * Return the type ID.
 */

guint vp_valuedict_get_type(void)
{
	static guint valuedict_type = 0;

	if (valuedict_type == 0)
	{
		static const GtkTypeInfo valuedict_info = {
			"VPValueDict",
			sizeof (VPValueDict),
			sizeof (VPValueDictClass),
			(GtkClassInitFunc)vp_valuedict_class_init,
			(GtkObjectInitFunc)vp_valuedict_init,
		};

		valuedict_type = gtk_type_unique(gtk_object_get_type(),&valuedict_info);
	}

	return valuedict_type;
}


/*
 * Initialise the class structure.
 */

static void vp_valuedict_class_init(VPValueDictClass *vdc)
{
}


/*
 * Initialise the instance structure.
 */

static void vp_valuedict_init(VPValueDict *this)
{
}


/*
 * Create a dictionary value.
 */

GtkObject *vp_valuedict_new(GtkObject *interp)
{
	VPValueDict *this;

	this = gtk_type_new(vp_valuedict_get_type());

	if ((this -> value.context = vp_interpreter_get_context(interp)) != NULL)
		this -> value.pyobj = pylibValueDictCtor(this -> value.context);

	return GTK_OBJECT(this);
}


/*
 * Set a key/value pair in a dictionary.
 */

void vp_valuedict_set(GtkObject *o,GtkObject *key,GtkObject *val)
{
	VPValueDict *this;
	VPValue *keyvi, *valvi;

	/* Type check the arguments. */

	g_return_if_fail(o != NULL);
	g_return_if_fail(VP_IS_VALUEDICT(o));

	this = VP_VALUEDICT(o);

	g_return_if_fail(key != NULL);
	g_return_if_fail(VP_IS_VALUE(key));

	keyvi = VP_VALUE(key);

	g_return_if_fail(val != NULL);
	g_return_if_fail(VP_IS_VALUE(val));

	valvi = VP_VALUE(val);

	/* Make sure the contexts are the same. */

	if (this -> value.context == keyvi -> context && this -> value.context == valvi -> context && this -> value.context != NULL)
		pylibValueDictSet(this -> value.context,this -> value.pyobj,keyvi -> pyobj,valvi -> pyobj);
}


/*****************************************************************************/
/* VPValueList                                                               */
/*****************************************************************************/

static void vp_valuelist_class_init(VPValueListClass *);
static void vp_valuelist_init(VPValueList *);


/*
 * Return the type ID.
 */

guint vp_valuelist_get_type(void)
{
	static guint valuelist_type = 0;

	if (valuelist_type == 0)
	{
		static const GtkTypeInfo valuelist_info = {
			"VPValueList",
			sizeof (VPValueList),
			sizeof (VPValueListClass),
			(GtkClassInitFunc)vp_valuelist_class_init,
			(GtkObjectInitFunc)vp_valuelist_init,
		};

		valuelist_type = gtk_type_unique(gtk_object_get_type(),&valuelist_info);
	}

	return valuelist_type;
}


/*
 * Initialise the class structure.
 */

static void vp_valuelist_class_init(VPValueListClass *vlc)
{
}


/*
 * Initialise the instance structure.
 */

static void vp_valuelist_init(VPValueList *this)
{
}


/*
 * Create a list value.
 */

GtkObject *vp_valuelist_new(GtkObject *interp)
{
	VPValueList *this;

	this = gtk_type_new(vp_valuelist_get_type());

	if ((this -> value.context = vp_interpreter_get_context(interp)) != NULL)
		this -> value.pyobj = pylibValueListCtor(this -> value.context);

	return GTK_OBJECT(this);
}


/*
 * Append a value to a list.
 */

void vp_valuelist_append(GtkObject *o,GtkObject *val)
{
	VPValueList *this;
	VPValue *valvi;

	/* Type check the arguments. */

	g_return_if_fail(o != NULL);
	g_return_if_fail(VP_IS_VALUELIST(o));

	this = VP_VALUELIST(o);

	g_return_if_fail(val != NULL);
	g_return_if_fail(VP_IS_VALUE(val));

	valvi = VP_VALUE(val);

	/* Make sure the contexts are the same. */

	if (this -> value.context == valvi -> context && this -> value.context != NULL)
		pylibValueListAppend(this -> value.context,this -> value.pyobj,valvi -> pyobj);
}
