/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This handles the GNOME implementation of the Dialog class.
 */


#include <Python.h>
#include <gnome.h>

#include "gvpmodint.h"


/* Extra information we need to keep with a dialog. */

struct dialogContext {
	PyObject	*pyobj;		/* The corresponding Python object. */
	int		buttMap[5];	/* Corresponding VP button value. */
};


static gboolean isModal(GtkWidget *);
static void handleClicked(GtkObject *,gint,gpointer);
static gboolean handleClose(GtkObject *,gpointer);


/*
 * The ctor.
 */

void *gvpDialogCtor(PyObject *po,void *parent,char *title,int buttons,char *continueText,int defaultButton,int modality,char *noText,char *yesText)
{
	const gchar *butts[6];
	gint b, defbutt;
	GtkWidget *gd;
	struct dialogContext *dcp;

	if ((dcp = Py_Malloc(sizeof (struct dialogContext))) == NULL)
		return NULL;

	dcp -> pyobj = po;

	/* Set the buttons, their text and the default. */

	b = 0;
	defbutt = -1;

	if (buttons & PYMOD_DIALOG_BUTTON_OK)
	{
		butts[b] = GNOME_STOCK_BUTTON_OK;
		dcp -> buttMap[b] = PYMOD_DIALOG_BUTTON_OK;

		if (defaultButton == PYMOD_DIALOG_BUTTON_OK)
			defbutt = b;

		++b;
	}

	if (buttons & PYMOD_DIALOG_BUTTON_YES)
	{
		butts[b] = (yesText != NULL ? yesText : GNOME_STOCK_BUTTON_YES);
		dcp -> buttMap[b] = PYMOD_DIALOG_BUTTON_YES;

		if (defaultButton == PYMOD_DIALOG_BUTTON_YES)
			defbutt = b;

		++b;
	}

	if (buttons & PYMOD_DIALOG_BUTTON_CONTINUE)
	{
		butts[b] = (continueText != NULL ? continueText : _("Continue"));
		dcp -> buttMap[b] = PYMOD_DIALOG_BUTTON_CONTINUE;

		if (defaultButton == PYMOD_DIALOG_BUTTON_CONTINUE)
			defbutt = b;

		++b;
	}

	if (buttons & PYMOD_DIALOG_BUTTON_NO)
	{
		butts[b] = (noText != NULL ? noText : GNOME_STOCK_BUTTON_NO);
		dcp -> buttMap[b] = PYMOD_DIALOG_BUTTON_NO;

		if (defaultButton == PYMOD_DIALOG_BUTTON_NO)
			defbutt = b;

		++b;
	}

	if (buttons & PYMOD_DIALOG_BUTTON_CANCEL)
	{
		butts[b] = GNOME_STOCK_BUTTON_CANCEL;
		dcp -> buttMap[b] = PYMOD_DIALOG_BUTTON_CANCEL;

		if (defaultButton == PYMOD_DIALOG_BUTTON_CANCEL)
			defbutt = b;

		++b;
	}

	butts[b] = NULL;

	if (defbutt < 0 && b > 0)
		defbutt = 0;

	/* Create the dialog. */

	gd = gnome_dialog_newv(title,butts);

	if (defbutt >= 0)
		gnome_dialog_set_default(GNOME_DIALOG(gd),defbutt);

	gvpDialogSetParent(gd,parent);

	if (modality == PYMOD_DIALOG_MODALITY_MODAL)
		gtk_window_set_modal(GTK_WINDOW(gd),TRUE);

	gtk_signal_connect(GTK_OBJECT(gd),"clicked",GTK_SIGNAL_FUNC(handleClicked),dcp);
	gtk_signal_connect(GTK_OBJECT(gd),"close",GTK_SIGNAL_FUNC(handleClose),dcp);

	gvpWidgetInit(gd,dcp);

	return gd;
}


/*
 * Add a child.
 */

void gvpDialogAddChild(void *self,void *child)
{
	GtkWidget *gd = (GtkWidget *)self;

	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(gd) -> vbox),(GtkWidget *)child,FALSE,FALSE,0);
}


/*
 * The method hide.
 */

void gvpDialogHide(void *self)
{
	GtkWidget *gd = (GtkWidget *)self;

	gtk_widget_hide(gd);

	if (isModal(gd))
		gtk_main_quit();
}


/*
 * The method show.
 */

void gvpDialogShow(void *self)
{
	GtkWidget *gd = (GtkWidget *)self;

	gtk_widget_show_all(gd);

	if (isModal(gd))
		gtk_main();
}


/*
 * Set the title property.
 */

int gvpDialogSetTitle(void *self,char *title)
{
	if (title != NULL)
		gtk_window_set_title((GtkWindow *)self,title);

	return 0;
}


/*
 * Set the parent of a dialog.
 */

void gvpDialogSetParent(GtkWidget *dlg,void *ptr)
{
	GtkWidget *parent;

	/* Find the top-level window of the parent. */
 
	if (ptr == NULL)
		return;

	parent = GTK_WIDGET(ptr);
 
	while (!GTK_IS_WINDOW(parent))
		if ((parent = parent -> parent) == NULL)
			return;
 
	gnome_dialog_set_parent(GNOME_DIALOG(dlg),GTK_WINDOW(parent));
}


/*
 * Handle a button click.
 */

static void handleClicked(GtkObject *self,gint gbutt,gpointer data)
{
	struct dialogContext *dcp = (struct dialogContext *)data;

	pymodSignalDialogClicked(dcp -> pyobj,dcp -> buttMap[gbutt]);
}


/*
 * Handle the dialog being closed.
 */

static gboolean handleClose(GtkObject *self,gpointer data)
{
	struct dialogContext *dcp = (struct dialogContext *)data;

	pymodSignalDialogClicked(dcp -> pyobj,0);

	/* In VP dialogs must be explicitly hidden or closed. */

	return TRUE;
}


/*
 * Return TRUE if the dialog is modal.
 */

static gboolean isModal(GtkWidget *gd)
{
	GtkArg arg;

	arg.name = "GtkWindow::modal";

	gtk_object_getv(GTK_OBJECT(gd),1,&arg);

	return GTK_VALUE_BOOL(arg);
}
