/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module handles application windows.
 */


#include <string.h>

#include "pylibint.h"


static signalDef **appWinSignals = NULL;	/* The application signals. */
static int (*signalEmitter)(pymodPyBase *,signalDef *,char *,...) = NULL;


static pylibAppWindow *findAppWinByName(const char *);
static pylibAppWindow *findAppWin(void *);


/*
 * Add an application window to the context, replacing any existing one of the
 * same name.
 */

void pylibAddAppWindow(struct pylibContext *context,void *ptr,const char *name)
{
	pylibAppWindow *aw;

	pylibRestoreContext(context);

	/* See if the name already exists. */

	if ((aw = findAppWinByName(name)) != NULL)
	{
		/*
		 * An existing one can only be changed if there isn't a
		 * corresponding Python ApplicationWindow.
		 */

		if (aw -> bw != NULL)
		{
			PyErr_Format(PyExc_RuntimeError,"An ApplicationWindow cannot be changed when in use by a script");
			pylibException();
			goto unlock;
		}

		aw -> window = ptr;

		goto unlock;
	}

	/* Create a new one. */

	if ((aw = Py_Malloc(sizeof (pylibAppWindow))) == NULL)
	{
		pylibException();
		goto unlock;
	}

	if (name == NULL)
		aw -> name = NULL;
	else if ((aw -> name = Py_Malloc(strlen(name) + 1)) == NULL)
	{
		pylibException();
		goto unlock;
	}
	else
		strcpy(aw -> name,name);

	aw -> window = ptr;
	aw -> bw = NULL;
	aw -> argv = NULL;

	aw -> next = context -> appWindows;
	context -> appWindows = aw;

unlock:
	pylibClearContext();
}


/*
 * Remove an application window from the context.
 */

void pylibRemoveAppWindow(struct pylibContext *context,void *ptr)
{
	pylibAppWindow *aw, **awp;

	pylibRestoreContext(context);

	/* Find the window. */

	for (awp = &context -> appWindows; (aw = *awp) != NULL; awp = &aw -> next)
		if (aw -> window == ptr || (ptr == NULL && aw -> name == NULL))
		{
			if (aw -> name == NULL)
			{
				PyErr_Format(PyExc_NameError,"Default ApplicationWindow cannot be removed");
				pylibException();
				goto unlock;
			}

			break;
		}

	if (aw == NULL)
	{
		PyErr_Format(PyExc_NameError,"Unknown ApplicationWindow");
		pylibException();
		goto unlock;
	}

	/* Do the remove. */

	if (aw -> bw != NULL)
	{
		PyErr_Format(PyExc_RuntimeError,"An ApplicationWindow cannot be removed when in use by a script");
		pylibException();
		goto unlock;
	}

	*awp = pylibDeleteAppWindow(aw);

unlock:
	pylibClearContext();
}
 

/*
 * Do most of the work of deleting an application window.  Return the one after
 * it in the linked list.
 */

pylibAppWindow *pylibDeleteAppWindow(pylibAppWindow *aw)
{
	pylibAppWindow *next = aw -> next;

	/*
	 * The reason for doing this is that it might result in calls to
	 * toolkit dtors that actually remove real widgets from the screen.
	 */
 
	if (aw -> bw != NULL)
		Py_DECREF(aw -> bw -> base.instance);

	Py_XDECREF(aw -> argv);

	if (aw -> name != NULL)
		Py_Free((ANY *)aw -> name);

	Py_Free((ANY *)aw);

	return next;
}

 
/*
 * Return the list of application windows for the current context.
 */
 
pylibAppWindow *pylibGetAppWindows(void)
{
	return pylibGetContext() -> appWindows;
}


/*
 * Set the application window's argv property.
 */

void pylibSetAppWindowArgv(struct pylibContext *context,void *ptr,int argc,char **argv)
{
	int i;
	pylibAppWindow *aw;
	PyObject *list;

	pylibRestoreContext(context);

	if ((aw = findAppWin(ptr)) == NULL)
	{
		pylibException();
		goto unlock;
	}

	/* Make sure it is being set before the control is created. */

	if (aw -> bw != NULL)
	{
		PyErr_Format(PyExc_TypeError,"ApplicationWindow.argv cannot be set when in use by a script");
		pylibException();
		goto unlock;
	}

	/* Create the list. */

	if ((list = PyList_New(argc)) == NULL)
	{
		pylibException();
		goto unlock;
	}

	for (i = 0; i < argc; ++i)
	{
		PyObject *arg;

		if ((arg = PyString_FromString(argv[i])) == NULL)
		{
			pylibException();
			goto unlock;
		}

		PyList_SET_ITEM(list,i,arg);
	}

	/* Remove any previous value, just in case. */

	Py_XDECREF(aw -> argv);

	/* Save the new value. */

	aw -> argv = list;

	/* If this is the default window then also set sys.argv. */

	if (ptr == NULL)
		PySys_SetArgv(argc,argv);

unlock:
	pylibClearContext();
}


/*
 * Define a set of signals for the application window.
 */

void pylibSetAppWindowSignals(struct pylibContext *context,char **signames)
{
	int nrsigs, i;
	signalDef *sigs, **sigtab;

	pylibRestoreContext(context);

	/*
	 * Make sure it is being set before any code has been executed (not
	 * just this context).
	 */

	if (pylibDoneFirstScript())
	{
		PyErr_Format(PyExc_RuntimeError,"Signals cannot be set once a script has been run");
		pylibException();
		goto unlock;
	}

	if (appWinSignals != NULL)
	{
		PyErr_Format(PyExc_RuntimeError,"Signals have already been set");
		pylibException();
		goto unlock;
	}

	/* Convert them to a signal structure array. */

	for (nrsigs = 0; signames[nrsigs] != NULL; ++nrsigs)
		;

	if (nrsigs == 0)
		goto unlock;

	if ((sigtab = Py_Malloc(sizeof (signalDef *) * nrsigs + 1)) == NULL)
	{
		pylibException();
		goto unlock;
	}

	if ((sigs = Py_Malloc(sizeof (signalDef) * nrsigs)) == NULL)
	{
		Py_Free((ANY *)sigtab);
		pylibException();
		goto unlock;
	}

	for (i = 0; i < nrsigs; ++i)
	{
		/*
		 * Converting to an object now means we are not depending on
		 * the application supplied names staying around.
		 */

		if ((sigs[i].nameobj = PyString_FromString(signames[i])) == NULL)
		{
			Py_Free((ANY *)sigtab);
			Py_Free((ANY *)sigs);
			pylibException();
			goto unlock;
		}

		sigs[i].name = PyString_AS_STRING(sigs[i].nameobj);

		sigtab[i] = &sigs[i];
	}

	sigtab[nrsigs] = NULL;

	appWinSignals = sigtab;

unlock:
	pylibClearContext();
}


/*
 * Emit an application window signal.
 */

void pylibEmitAppWindowSignal(struct pylibContext *context,void *ptr,char *signame,PyObject *val)
{
	pylibAppWindow *aw;
	signalDef **sigtab, *sig;

	pylibRestoreContext(context);

	if ((aw = findAppWin(ptr)) == NULL)
	{
		pylibException();
		goto unlock;
	}

	/* Find the signal structure. */

	if ((sigtab = pylibGetAppWindowSignals()) == NULL)
		sig = NULL;
	else
		while ((sig = *sigtab++) != NULL)
			if (strcmp(sig -> name,signame) == 0)
				break;

	if (sig == NULL)
	{
		PyErr_Format(PyExc_AttributeError,"Unknown ApplicationWindow signal");
		pylibException();
		goto unlock;
	}

	/*
	 * Do nothing if there is no source for the signal or there is no
	 * emitter function.  (The latter is just a sanity check.)
	 */

	if (aw -> bw == NULL || signalEmitter == NULL)
		goto unlock;

	/* Emit the signal. */

	if (val == NULL)
		val = Py_None;

	(*signalEmitter)(aw -> bw,sig,"(O)",val);

unlock:
	pylibClearContext();
}


/*
 * Set the signal emitter function.
 */

void pylibSetSignalEmitter(int (*emitter)(pymodPyBase *,signalDef *,char *,...))
{
	signalEmitter = emitter;
}


/*
 * Return the array of application defined signals.
 */

signalDef **pylibGetAppWindowSignals(void)
{
	return appWinSignals;
}


/*
 * Look for the application window.
 */

static pylibAppWindow *findAppWin(void *ptr)
{
	pylibAppWindow *aw;

	for (aw = pylibGetAppWindows(); aw != NULL; aw = aw -> next)
		if (aw -> window == ptr || (ptr == NULL && aw -> name == NULL))
			break;

	if (aw == NULL)
		PyErr_Format(PyExc_NameError,"Unknown ApplicationWindow");

	return aw;
}


/*
 * Look for the application window by its name.
 */

static pylibAppWindow *findAppWinByName(const char *name)
{
	pylibAppWindow *aw;

	for (aw = pylibGetAppWindows(); aw != NULL; aw = aw -> next)
		if ((name == NULL && aw -> name == NULL) || (name != NULL && aw -> name != NULL && strcmp(name,aw -> name) == 0))
			break;

	return aw;
}
