/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module handles the initialisation and termination of Python.
 */


#include "pylibint.h"


static int nrInterps;			/* The nr. of interpreters. */
static PyThreadState *mainThread;	/* The main thread. */
static struct pylibContext *currentContext;	/* The current context. */


static void libError(void (*)(const char *,void *),const char *,void *);


/*
 * Initialise the interpreter.
 */

struct pylibContext *pylibInit(void (*eh)(const char *,void *),void *tkData)
{
	struct pylibContext *context;
	PyThreadState *newThread;

	/* Do the right initialisation. */

	if (Py_IsInitialized())
	{
		PyEval_RestoreThread(mainThread);

		if ((newThread = Py_NewInterpreter()) == NULL)
		{
			PyErr_Print();
			libError(eh,"Py_NewInterpreter failed when creating Python context",tkData);

			PyEval_SaveThread();

			return NULL;
		}
	}
	else
	{
		Py_Initialize();
		newThread = mainThread = PyThreadState_Get();
		nrInterps = 0;
		currentContext = NULL;
	}

	/* Create and initialise the context. */

	if ((context = Py_Malloc(sizeof (struct pylibContext))) == NULL)
	{
		PyErr_Print();
		libError(eh,"Py_Malloc failed when creating Python context",tkData);

		if (newThread != mainThread)
			Py_EndInterpreter(newThread);
		else
			Py_Finalize();

		PyEval_SaveThread();

		return NULL;
	}

	context -> eHandler = eh;
	context -> thread = newThread;
	context -> appWindows = NULL;
	context -> ranScript = 0;
	context -> tkData = tkData;

	++nrInterps;

	PyEval_SaveThread();

	return context;
}


/*
 * Tidy up the interpreter.
 */

void pylibDone(struct pylibContext *context,char *postcode)
{
	pylibAppWindow *aw;

	pylibRestoreContext(context);

	/* Run any post-code. */

	if (context -> ranScript && postcode && PyRun_SimpleString(postcode) != 0)
		pylibException();

	/* Release the list of application windows. */

	while ((aw = context -> appWindows) != NULL)
		context -> appWindows = pylibDeleteAppWindow(aw);

	if (context -> thread != mainThread)
		Py_EndInterpreter(context -> thread);
	else if (nrInterps > 1)
	{
		/*
		 * Note that this means that the main interpreter will never be
		 * removed.
		 */

		PyErr_Format(PyExc_RuntimeError,"Destroying main interpreter while sub-interpreters still exist");
		pylibException();
	}
	else
		Py_Finalize();

	Py_Free((ANY *)context);

	--nrInterps;

	pylibClearContext();
}

 
/*
 * Acquire the Python global lock and make the given context current.
 */
 
void pylibRestoreContext(struct pylibContext *context)
{
	/* Panic if we don't have a context. */

	if (context == NULL)
		Py_FatalError("trying to set empty Python context");

	PyEval_RestoreThread(context -> thread);
	currentContext = context;
}
 
 
/*
 * Get the current context.
 */
 
struct pylibContext *pylibGetContext(void)
{
	return currentContext;
}


/*
 * Clear the current context and release the Python global lock.
 */
 
void pylibClearContext(void)
{
	currentContext = NULL;
 
	PyEval_SaveThread();
}


/*
 * Return the current context and clear it.
 */

struct pylibContext *pylibGetAndClearContext(void)
{
	struct pylibContext *old = pylibGetContext();

	pylibClearContext();

	return old;
}
 
 
/*
 * Call the current context's error handler when an exception has been raised.
 */

void pylibException(void)
{
	struct pylibContext *thisContext;

	PyErr_Print();

	thisContext = pylibGetAndClearContext();
	libError(thisContext -> eHandler,"A Python exception was raised",thisContext -> tkData);
	pylibRestoreContext(thisContext);
}


/*
 * Call an error handler with an error message.
 */

static void libError(void (*eh)(const char *,void *),const char *emess,void *tkData)
{
	(*eh)(emess,tkData);
}
