/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module implements the Python part of the ApplicationWindow class.
 */


#include <stddef.h>
#include <string.h>

#include "pymodint.h"


static int applicationWindowCtor(pymodPyBase *,PyObject *,PyObject *);
static PyObject *applicationWindowUpdate(pymodPyBase *,PyObject *);
static PyObject *applicationWindowGetNames(pymodPyBase *,PyObject *,PyObject *);


/*****************************************************************************/
/* Describe the ApplicationWindow class.                                     */
/*****************************************************************************/

/* The wrapped toolkit object. */

static PyTypeObject applicationWindowType = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,
	"__vpApplicationWindow__",
	sizeof (pymodPyApplicationWindow),
	0,
	pymodDeallocApplicationWindow,
};

/* The methods of this class. */

static PyMethodDef applicationWindowMethods[] = {
	{"update", (PyCFunction)applicationWindowUpdate, METH_VARARGS, },
	{NULL}
};

/* The functions of this class. */

static PyMethodDef applicationWindowFunctions[] = {
	{"getNames", (PyCFunction)applicationWindowGetNames, METH_VARARGS|METH_KEYWORDS, },
	{NULL}
};

/* The properties of this class. */

propDef pymodPropApplicationWindowArgv = {
	"argv",
	stringProp,
	offsetof(pymodPyApplicationWindow,applicationWindow.argv),
	PROP_FLAG_LIST|PROP_FLAG_NOT_CTOR,
};

propDef pymodPropApplicationWindowName = {
	"name",
	stringProp,
	offsetof(pymodPyApplicationWindow,applicationWindow.name),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
};

static classPropDef applicationWindowProperties[] = {
	{&pymodPropApplicationWindowArgv, },
	{&pymodPropContainerChildren, },
	{&pymodPropApplicationWindowName, },
	{NULL}
};

/* The class definition. */
 
classDef pymodClassApplicationWindow = {
	"ApplicationWindow",
	applicationWindowProperties,
	applicationWindowMethods,
	NULL,
	NULL,
	applicationWindowCtor,
	&applicationWindowType,
	applicationWindowFunctions,
};


/*
 * The class ctor.
 */
 
static int applicationWindowCtor(pymodPyBase *self,PyObject *args,PyObject *kwds)
{
        pymodDataApplicationWindow *awd = &((pymodPyApplicationWindow *)self) -> applicationWindow;
	pylibAppWindow *aw;
	pymodColorDef cd;
 
        if (pymodParseArgs(&pymodClassApplicationWindow,self,args,kwds,applicationWindowProperties,"c")
< 0)
                return -1;

	/* Search for the right application window to use. */

	for (aw = pylibGetAppWindows(); aw != NULL; aw = aw -> next)
	{
		if (awd -> name == NULL && aw -> name == NULL)
			break;

		if (awd -> name != NULL && aw -> name != NULL && strcmp(PyString_AS_STRING(awd -> name),aw -> name) == 0)
			break;
	}

	if (aw == NULL)
	{
		if (awd -> name == NULL)
			PyErr_Format(PyExc_SystemError,"Internal error: unable to get default application window");
		else
			PyErr_Format(PyExc_NameError,"There is no ApplicationWindow with the name %s",PyString_AS_STRING(awd -> name));

		return -1;
	}

	/* Check an object doesn't already exist for this window. */

	if (aw -> bw != NULL)
	{
		if (awd -> name == NULL)
			PyErr_Format(PyExc_SystemError,"The default ApplicationWindow has already been created");
		else
			PyErr_Format(PyExc_NameError,"The ApplicationWindow with the name %s has already been created",PyString_AS_STRING(awd -> name));

		return -1;
	}

	aw -> bw = (pymodPyBase *)self;

	/* Save the toolkit pointer. */

	awd -> container.control.base.tkPtr = aw -> window;
	awd -> aw = aw;

	/* Handle argv. */

	if (aw -> argv != NULL)
	{
		awd -> argv = aw -> argv;
		Py_INCREF(awd -> argv);
	}
	else if ((awd -> argv = PyList_New(0)) == NULL)
		return -1;

	/* Get the background from the driver. */

	if ((*((pymodApplicationWindowDrivers *)pymodClassApplicationWindow.drivers) -> getBackground)(
			awd -> container.control.base.tkPtr,
			&cd) < 0)
		return -1;

	/* Wrap the application supplied color. */

	if ((awd -> container.control.background = pymodColorNew(&cd)) == NULL)
		return -1;

	return 0;
}


/*
 * Delete a Python application window object.
 */
 
void pymodDeallocApplicationWindow(PyObject *self)
{
	/* The application window is no longer in use. */

        ((pymodPyApplicationWindow *)self) -> applicationWindow.aw -> bw = NULL;
 
        pymodDeallocContainer(self);
}


/*
 * The method update.
 */
 
static PyObject *applicationWindowUpdate(pymodPyBase *self,PyObject *args)
{
	pymodPyApplicationWindow *aww;
	PyObject *children;
 
	if (pymodParseArgs(&pymodClassApplicationWindow,self,args,NULL,NULL,
				"mupdate",
				&aww) < 0)
		return NULL;
 
	/* Find the child to update. */

	if ((children = aww -> applicationWindow.container.children) != NULL)
	{
		int i;

		for (i = 0; i < PyList_GET_SIZE(children); ++i)
		{
			pymodPyBase *child = (pymodPyBase *)PyList_GET_ITEM(children,i);

			if (child -> base.cd != &pymodClassTopLevel && child -> base.tkPtr != NULL)
				(*((pymodApplicationWindowDrivers *)pymodClassApplicationWindow.drivers) -> update)(
					aww -> applicationWindow.container.control.base.tkPtr,
					child -> base.tkPtr);
		}
	}

	Py_INCREF(Py_None);
	return Py_None;
}


/*
 * The getNames function.
 */

static PyObject *applicationWindowGetNames(pymodPyBase *self,PyObject *args,PyObject *kwds)
{
	int len, i;
	pylibAppWindow *aw;
	PyObject *list;

	if (pymodParseArgs(&pymodClassApplicationWindow,self,args,kwds,NULL,
				"sgetNames") < 0)
		return NULL;

	/* Create a list of the right length. */

	len = 0;

	for (aw = pylibGetAppWindows(); aw != NULL; aw = aw -> next)
		if (aw -> name != NULL)
			++len;

	if ((list = PyList_New(len)) == NULL)
		return NULL;

	/* Now put the named names into the list. */

	i = 0;

	for (aw = pylibGetAppWindows(); aw != NULL; aw = aw -> next)
		if (aw -> name != NULL)
		{
			PyObject *nameobj;

			if ((nameobj = PyString_FromString(aw -> name)) == NULL)
			{
				Py_DECREF(list);
				return NULL;
			}

			PyList_SET_ITEM(list,i,nameobj);

			++i;
		}

	return list;
}
