/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module manages the VP classes.
 */


#include <stdio.h>
#include <stddef.h>
#include <string.h>

#include "pymodint.h"


/*
 * The table of all the class definitions.  The order is important - a class
 * must appear in the table before all of its super-classes.
 */

static classDef *classTable[] = {
	&pymodClassColor,
	&pymodClassPieSlice,
	&pymodClassPieChart,
	&pymodClassGraphPoint,
	&pymodClassGraphSeries,
	&pymodClassGraph,
	&pymodClassChart,
	&pymodClassLabel,
	&pymodClassGrid,
	&pymodClassHBox,
	&pymodClassVBox,
	&pymodClassMessage,
	&pymodClassDialog,
	&pymodClassTopLevel,
	&pymodClassApplicationWindow,
	&pymodClassContainer,
	&pymodClassControl,
	NULL
};


/* Strings we convert to objects once. */

static PyObject *vpWrapper = NULL;


static int convertProperties(classDef *);
static int convertEnums(classDef *);
static int convertSignals(classDef *);
static int convertMethods(PyObject *,PyMethodDef *,PyObject *);
pymodPyBase *createWrapperObject(PyInstanceObject *,classDef *);


/*
 * Initialise the classes in the given module dictionary.
 */

int pymodInitClasses(PyObject *moddict)
{
	classDef **cdp, *cd;

	/* Convert some strings. */

	if (vpWrapper == NULL)
	{
		if ((vpWrapper = PyString_FromString("__vpWrapper__")) == NULL)
			return -1;
	}

	/*
	 * Add any application supplied signals, and tell it how to emit
	 * signals.
	 */

	pymodClassApplicationWindow.signals = pylibGetAppWindowSignals();
	pylibSetSignalEmitter(pymodHandleSignal);

	/* Initialise each of the classes. */

	for (cdp = classTable; (cd = *cdp) != NULL; ++cdp)
	{
		PyObject *cls;

		/* Get the Python class object. */

		if ((cls = PyDict_GetItemString(moddict,cd -> name)) == NULL)
			return -1;

		/* Make sure it really is a class. */

		if (!PyClass_Check(cls))
		{
			PyErr_Format(PyExc_SystemError,"Internal error: vp.%s should be a class",cd -> name);

			return -1;
		}

		cd -> classobj = cls;

		/* Convert any class properties to objects. */

		if (cd -> props != NULL && convertProperties(cd) < 0)
			return -1;

		/* Convert any class enums to objects. */

		if (cd -> enums != NULL && convertEnums(cd) < 0)
			return -1;

		/* Convert any class signals to objects. */

		if (cd -> signals != NULL && convertSignals(cd) < 0)
			return -1;

		/*
		 * Add any class methods in case the class.method(self,) style
		 * is used.
		 */

		if (cd -> methods != NULL && convertMethods(((PyClassObject *)cd -> classobj) -> cl_dict,cd -> methods,NULL) < 0)
			return -1;

		/* Add any class functions. */

		if (cd -> funcs != NULL && convertMethods(((PyClassObject *)cd -> classobj) -> cl_dict,cd -> funcs,NULL) < 0)
			return -1;

		/* Any other class specific initialisation. */

		if (cd -> init != NULL && (*cd -> init)(cd) < 0)
			return -1;
	}

	/* No errors. */

	return 0;
}


/*
 * Convert a class's raw properties.  Return -1 if there was an error.
 */

static int convertProperties(classDef *cd)
{
	classPropDef *cpd;

	for (cpd = cd -> props; cpd -> prop != NULL; ++cpd)
	{
		/*
		 * The same property can appear in many different classes if it
		 * is inherited, so check this hasn't already been done.
		 */

		if (cpd -> prop -> nameobj != NULL)
			continue;

		if ((cpd -> prop -> nameobj = PyString_FromString(cpd -> prop -> name)) == NULL)
			return -1;
	}

	return 0;
}


/*
 * Convert a class's raw enums.  Return -1 if there was an error.
 */

static int convertEnums(classDef *cd)
{
	enumDef **edp, *ed;

	for (edp = cd -> enums; (ed = *edp) != NULL; ++edp)
	{
		enumValueDef *evd;

		/* Add each value. */

		for (evd = ed -> values; evd -> name != NULL; ++evd)
		{
			enumObject *eo;

			/* Create the enum object. */

			if ((eo = PyObject_NEW(enumObject,ed -> type)) == NULL)
				return -1;

			/* Initialise it. */

			eo -> evd = evd;
			eo -> cd = cd;

			/* Add it to the class dictionary. */

			if (PyDict_SetItemString(((PyClassObject *)cd -> classobj) -> cl_dict,evd -> name,(PyObject *)eo) < 0)
				return -1;

			/*
			 * Keep a pointer to the object so we can easily do int
			 * to object conversions.  We don't Py_DECREF() after
			 * the PyDict_SetItemString() above to ensure that this
			 * pointer is always valid.
			 */

			evd -> objval = (PyObject *)eo;
		}
	}

	return 0;
}


/*
 * Convert a class's raw signals.  Return -1 if there was an error.
 */

static int convertSignals(classDef *cd)
{
	signalDef **sdp, *sd;

	for (sdp = cd -> signals; (sd = *sdp) != NULL; ++sdp)
	{
		signalObject *so;

		/*
		 * Convert the name to an object if it hasn't already been
		 * done.  (Application defined windows will have already done
		 * it.)
		 */

		if (sd -> nameobj == NULL && (sd -> nameobj = PyString_FromString(sd -> name)) == NULL)
			return -1;

		/* Create the signal object. */

		if ((so = PyObject_NEW(signalObject,&pymodSignalType)) == NULL)
			return -1;

		/* Initialise it. */

		so -> sd = sd;
		so -> cd = cd;

		/* Add it to the class dictionary. */

		if (PyDict_SetItem(((PyClassObject *)cd -> classobj) -> cl_dict,sd -> nameobj,(PyObject *)so) < 0)
			return -1;
	}

	return 0;
}


/*
 * Convert a raw methods.  Return -1 if there was an error.
 */

static int convertMethods(PyObject *dict,PyMethodDef *md,PyObject *inst)
{
	while (md -> ml_name != NULL)
	{
		PyObject *meth;

		/* Create the function. */

		if ((meth = PyCFunction_New(md,inst)) == NULL)
			return -1;

		/* Add it to the dictionary. */

		if (PyDict_SetItemString(dict,md -> ml_name,meth) < 0)
			return -1;

		Py_DECREF(meth);

		++md;
	}

	return 0;
}


/*
 * Called by the VP class ctors.  The arguments should be a tuple of the
 * instance being created and a dictionary of properties.
 */

PyObject *pymodCallCtor(PyObject *ignore,PyObject *args)
{
	classDef **cdp, *cd;
	PyInstanceObject *inst;
	PyObject *cls;
	pymodPyBase *wo;

	/* Check the arguments are what we expect. */

	if (!PyTuple_Check(args) || PyTuple_GET_SIZE(args) != 3 ||
	    !PyInstance_Check(PyTuple_GET_ITEM(args,0)))
	{
		PyErr_Format(PyExc_SystemError,"Internal error: invalid arguments to __vpCallCtor__");

		return NULL;
	}

	inst = (PyInstanceObject *)PyTuple_GET_ITEM(args,0);

	/* Search the class table. */

	cls = (PyObject *)inst -> in_class;

	for (cdp = classTable; (cd = *cdp) != NULL; ++cdp)
		if (PyClass_IsSubclass(cls,cd -> classobj))
			break;

	if (cd == NULL)
	{
		PyErr_Format(PyExc_SystemError,"Internal error: non-VP class called __vpCallCtor__");

		return NULL;
	}

	/* Check if it's an abstract class. */

	if (cd -> ctor == NULL)
	{
		PyErr_Format(PyExc_TypeError,"%s is an abstract VP class",cd -> name);

		return NULL;
	}

	/* Create the wrapper. */

	if ((wo = createWrapperObject(inst,cd)) == NULL)
		return NULL;

	/* Finally, create the toolkit object. */

	if ((*cd -> ctor)(wo,PyTuple_GET_ITEM(args,1),PyTuple_GET_ITEM(args,2)) < 0)
		return NULL;

	Py_INCREF(Py_None);
	return Py_None;
}


/*
 * Get the wrapped object from an instance's dictionary.  Return NULL if there
 * was an error - either because it's a user defined class and they forgot to
 * call the super-class ctor, or there was an internal error.
 */

pymodPyBase *pymodGetWrappedObject(PyObject *inst)
{
	pymodPyBase *wo;

	if ((wo = (pymodPyBase *)PyDict_GetItem((PyObject *)(((PyInstanceObject *)inst) -> in_dict),vpWrapper)) == NULL)
	{
		PyErr_Format(PyExc_RuntimeError,"Unable to find the underlying toolkit object");

		return NULL;
	}

	return wo;
}


/*
 * The dealloc function for the Base class.
 */

void pymodDeallocBase(PyObject *self)
{
	pymodDataBase *wd = &((pymodPyBase *)self) -> base;

	/* Disconnect all signals. */

	while (wd -> connections != NULL)
	{
		connectionDef *cd;

		cd = wd -> connections;
		wd -> connections = cd -> next;

		Py_DECREF(cd -> slot);

		Py_Free((ANY *)cd);
	}

	/* Delete the properties. */

	pymodPropsDeleteSelf((pymodPyBase *)self);

	/*
	 * If there is a toolkit object and there is a toolkit dtor, call it.
	 */

	if (wd -> tkPtr != NULL && wd -> cd -> drivers != NULL && wd -> cd -> drivers -> guidtor != NULL)
		(*wd -> cd -> drivers -> guidtor)(wd -> tkPtr);
}


/*
 * Create the wrapper object and add it to the instance.  Complete the setting
 * up of the instance.
 */

pymodPyBase *createWrapperObject(PyInstanceObject *inst,classDef *cd)
{
	pymodPyBase *wo;

	if ((wo = PyObject_NEW(pymodPyBase,cd -> pytype)) == NULL)
		return NULL;

	/*
	 * Initialiase the non-Python parts of the wrapper object.  This
	 * ensures that any PyObject pointers are NULL so we can do Py_XDECREF
	 * safely.
	 */

	memset(&wo -> base,0,cd -> pytype -> tp_basicsize - offsetof(pymodPyBase,base));

	/* Initialise the remaining common parts of the wrapper. */

	wo -> base.cd = cd;
	wo -> base.instance = (PyObject *)inst;

	/* Add the wrapper object to the instance dictionary. */

	if (PyDict_SetItem(inst -> in_dict,vpWrapper,(PyObject *)wo) < 0)
	{
		Py_DECREF(wo);
		return NULL;
	}

	Py_DECREF(wo);

	/*
	 * Add any class methods to the instance.  Note that we are binding the
	 * wrapped object rather than the instance - this saves a dictionary
	 * lookup whenever the method is called using the self.method() style.
	 */

	if (cd -> methods != NULL && convertMethods(inst -> in_dict,cd -> methods,(PyObject *)wo) < 0)
		return NULL;

	return wo;
}
