/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module implements the Python part of the Color class.
 */


#include <stddef.h>
#include <string.h>
#include <ctype.h>

#include "pymodint.h"


static int colorInit(classDef *);
static int colorCtor(pymodPyBase *,PyObject *,PyObject *);
static int colorSetRGB(pymodPyBase *);
static PyObject *colorRepr(pymodPyColor *);


/*****************************************************************************/
/* Describe the Color class.                                                 */
/*****************************************************************************/

/* The wrapped toolkit object. */

static PyTypeObject colorType = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,
	"__vpColor__",
	sizeof (pymodPyColor),
	0,
	pymodDeallocColor,
	0,
	0,
	0,
	0,
	(reprfunc)colorRepr,
};

/* The properties of this class. */

propDef pymodPropColorRGB = {
	"rgb",
	stringProp,
	offsetof(pymodPyColor,color.rgb),
};

static classPropDef colorProperties[] = {
	{&pymodPropColorRGB, colorSetRGB, },
	{NULL}
};


/* The class definition. */
 
classDef pymodClassColor = {
	"Color",
	colorProperties,
	NULL,
	NULL,
	NULL,
	colorCtor,
	&colorType,
	NULL,
	colorInit,
};


static int convertColor(pymodDataColor *);
static PyObject *colorToString(pymodColorDef *);


/*
 * The class initialiser.
 */

static int colorInit(classDef *cd)
{
	struct colorTab {
		char	*name;
		char	*color;
	};

	static struct colorTab initColors[] = {
		{"black",	"#000000000000"},
		{"white",	"#ffffffffffff"},
		{"darkGray",	"#800080008000"},
		{"gray",	"#a000a000a400"},
		{"lightGray",	"#c000c000c000"},
		{"red",		"#ffff00000000"},
		{"green",	"#0000ffff0000"},
		{"blue",	"#00000000ffff"},
		{"cyan",	"#0000ffffffff"},
		{"magenta",	"#ffff0000ffff"},
		{"yellow",	"#ffffffff0000"},
		{"darkRed",	"#800000000000"},
		{"darkGreen",	"#000080000000"},
		{"darkBlue",	"#000000008000"},
		{"darkCyan",	"#000080008000"},
		{"darkMagenta",	"#800000008000"},
		{"darkYellow",	"#800080000000"},
		{NULL}
	};

	struct colorTab *ct;

	/* Add the pre-defined colors to the class dictionary. */

	for (ct = initColors; ct -> name != NULL; ++ct)
	{
		PyObject *inst, *args;

		/* Create the instance. */

		if ((args = Py_BuildValue("(s)",ct -> color)) == NULL)
			return -1;

		if ((inst = PyInstance_New(cd -> classobj,args,NULL)) == NULL)
			return -1;

		Py_DECREF(args);

		/* Add it to the class dictionary. */

		if (PyDict_SetItemString(((PyClassObject *)cd -> classobj) -> cl_dict,ct -> name,inst) < 0)
			return -1;

		Py_DECREF(inst);
	}

	return 0;
}


/*
 * The class ctor.
 */

static int colorCtor(pymodPyBase *self,PyObject *args,PyObject *kwds)
{
	pymodDataColor *cd = &((pymodPyColor *)self) -> color;

	if (pymodParseArgs(&pymodClassColor,self,args,kwds,colorProperties,"c") < 0)
		return -1;

	/* Convert the color from a string to an RGB value. */

	if (convertColor(cd) < 0)
		return -1;

	/*
	 * Call the driver.
	 *
	 * color is the color.
	 */

	if ((cd -> base.tkPtr = (*((pymodColorDrivers *)pymodClassColor.drivers) -> ctor)(
			(PyObject *)self,
			&cd -> color)) == NULL)
	{
		pymodBadToolkitCtor(pymodClassColor.name);
		return -1;
	}

	return 0;
}


/*
 * Handle the setting of the rgb property.
 */

static int colorSetRGB(pymodPyBase *self)
{
	pymodDataColor *cd = &((pymodPyColor *)self) -> color;

	if (convertColor(cd) < 0)
		return -1;

	return (*((pymodColorDrivers *)pymodClassColor.drivers) -> setRGB)(cd -> base.tkPtr,&cd -> color);
}


/*
 * Convert a color from a string to the internal (GUI independent) RGB format.
 */

#define	xch2int(c)	(isdigit(c) ? ((c) - '0') : (tolower(c) - 'a' + 10))

static int convertColor(pymodDataColor *cd)
{
	char *cstr, *roff, *goff, *boff;
	int i, nch, invalid = 1;
	pymodColorDef *col;

	cstr = PyString_AS_STRING(cd -> rgb);
	col = &cd -> color;

	/* Check the prefix. */

	if (*cstr++ == '#')
	{
		char *cp;

		/* Check it is all hex. */

		for (cp = cstr; *cp != '\0'; ++cp)
			if (!isxdigit(*cp))
				break;

		if (*cp == '\0')
			switch (strlen(cstr))
			{
			case 3:
				roff = &cstr[0];
				goff = &cstr[1];
				boff = &cstr[2];
				nch = 1;
				invalid = 0;
				break;

			case 6:
				roff = &cstr[0];
				goff = &cstr[2];
				boff = &cstr[4];
				nch = 2;
				invalid = 0;
				break;

			case 12:
				roff = &cstr[0];
				goff = &cstr[4];
				boff = &cstr[8];
				nch = 4;
				invalid = 0;
				break;
			}
	}

	if (invalid)
	{
		PyErr_Format(PyExc_TypeError,"The format of a color string is either '#rgb', '#rrggbb' or '#rrrrggggbbbb' where r, g and b are hexadecimal values");
		return -1;
	}

	/* Do the conversion now we know where all the bits are. */

	col -> bitsPerColor = nch * 4;
	col -> red = col -> green = col -> blue = 0;

	for (i = 0; i < nch; ++i)
	{
		col -> red = (col -> red << 4) | (unsigned short)xch2int(*roff);
		col -> green = (col -> green << 4) | (unsigned short)xch2int(*goff);
		col -> blue = (col -> blue << 4) | (unsigned short)xch2int(*boff);

		++roff;
		++goff;
		++boff;
	}

	while (i++ < 4)
	{
		col -> red <<= 4;
		col -> green <<= 4;
		col -> blue <<= 4;
	}

	return 0;
}


/*
 * The repr() function for a color.
 */

static PyObject *colorRepr(pymodPyColor *cw)
{
	return colorToString(&cw -> color.color);
}


/*
 * Convert a color definition to its string representation.  Return NULL is
 * there was an error.
 */

static PyObject *colorToString(pymodColorDef *col)
{
	char buf[20], *fmt;
	unsigned short red, green, blue;

	switch (col -> bitsPerColor)
	{
	case 4:
		fmt = "#%0.1hx%0.1hx%0.1hx";
		red = col -> red >> 12;
		green = col -> green >> 12;
		blue = col -> blue >> 12;
		break;

	case 8:
		fmt = "#%0.2hx%0.2hx%0.2hx";
		red = col -> red >> 8;
		green = col -> green >> 8;
		blue = col -> blue >> 8;
		break;

	case 16:
		fmt = "#%0.4hx%0.4hx%0.4hx";
		red = col -> red;
		green = col -> green;
		blue = col -> blue;
		break;

	default:
		PyErr_Format(PyExc_SystemError,"Internal error: invalid number of bits per color");
		return NULL;
	}

	sprintf(buf,fmt,red,green,blue);

	return PyString_FromString(buf);
}


/*
 * Create a new color from a color definition.  Return NULL if there was an
 * error.
 */

pymodPyColor *pymodColorNew(pymodColorDef *cd)
{
	PyObject *args, *inst;

	/* Turn it into an argument list for the ctor. */

	if ((args = Py_BuildValue("(N)",colorToString(cd))) == NULL)
		return NULL;

	/*
	 * Wrap it.  Note we throw away the only pointer to the instance so
	 * this object will never be garbage collected.
	 */

	inst = PyInstance_New(pymodClassColor.classobj,args,NULL);

	Py_DECREF(args);

	if (inst == NULL)
		return NULL;

	return (pymodPyColor *)pymodGetWrappedObject(inst);
}
