/*
 * This module handles attribute values.
 */


#include "gdchartint.h"


/*
 * These are definitions of the attribute values.
 *
 * gdc_generate_image (default: TRUE)
 *	Set if the GD image should be converted to a PNG image.
 *
 * gdc_hold_img (default: FALSE)
 *	Set if the chart should be drawn on an existing GD image (which is not
 *	destroyed afterwards).
 *
 * gdc_image (default: NULL)
 *	If gdc_hold_img is set then this points to the GD image of the drawn
 *	chart.  If it was originally NULL then a new GD image is created,
 *	otherwise the chart is drawn on this existing image.
 *
 * gdc_BGColor (default: black)
 *	The background color.
 *
 * gdc_transparent_bg (default: TRUE)
 *	Set to make the background color transparent.
 *
 * gdc_title (default: NULL)
 *	The text of the chart's optional title.  It may include '\n's.
 *
 * gdc_title_size (default: medium bold)
 *	The font to use for the chart's title.
 *
 * gdc_TitleColor (default: the default color)
 *	The color of the chart's title.
 *
 * gdc_label_size (default: small)
 *	The font to use for labels.  For graphs the fonts used for the x and
 *	y axis labels may be explicitly set using gdccht_xaxisfont_size and
 *	gdccht_yaxisfont_size.
 *
 * gdc_LabelColor (default: the default color)
 *	The color to use for labels.  For graphs the colors used for the x and
 *	y axis labels may be explicitly set using gdccht_XLabelColor,
 *	gdccht_YLabelColor and GDC_YLabel2Color.
 *
 * gdc_LineColor (default: the default color)
 *	The color used to draw lines.
 *
 * gdc_3d_angle (default: 45)
 *	For 3D pie charts this is the position of the 3D shadow where North is
 *	0 and 360, East is 90, South is 180 and West is 270.  For graphs it
 *	should be in the range 1-89 and determines, in a limited way, the
 *	viewer's position relative to the graph - the higher the value, the
 *	higher the viewer appears to be.
 *
 * gdc_3d_depth (default: 5)
 *	The depth of the 3D shadow expressed as a %age of the chart size.
 *
 * gdcpie_EdgeColor (default: edging is disabled)
 *	The color used to draw a line around the outside of the slices of a pie
 *	chart.
 *
 * gdcpie_label_dist (default: 100)
 *	The distance between a label and the corresponding slice expressed as a
 *	%age of the height of the GDC_label_size font.  It may be negative.
 *
 * gdcpie_label_line (default: FALSE)
 *	Set if a line should be drawn between a slice's label and the slice
 *	itself.
 *
 * gdcpie_percent_labels (default: GDCPIE_PCT_NONE)
 *	Specifies where the slice %age (if any) is placed relative to the
 *	label, either above it (GDCPIE_PCT_ABOVE), below it (GDCPIE_PCT_BELOW),
 *	to the left of it (GDCPIE_PCT_LEFT), to the right of it
 *	(GDCPIE_PCT_RIGHT), or not at all (GDCPIE_PCT_NONE).
 *
 * gdcpie_other_threshold
 *	Don't understand this - don't use it.
 *
 * gdccht_stack_type (default: GDC_STACK_BEHIND)
 *	Specifies how multiple series are drawn relaive to each other.  The
 *	actual effects vary according to the type of the graph, and they are
 *	not always consistent (and sometimes wrong).  GDC_STACK_BEHIND means
 *	that each series is drawn in a different plane, the first series being
 *	at the front and the last at the back.  GDC_STACK_BESIDE means that all
 *	series are drawn in the same plane.  GDC_STACK_LAYERED means that all
 *	series are drawn in the same plane and the values are stacked on top of
 *	each other.
 *
 * gdccht_bar_width (default: 75)
 *	Specifies the %age of the horizontal space allocated to the bars in a
 *	3D bar chart are actually occupied by the bars.  A value of 100 means
 *	there is no gap between bars.
 *
 * gdccht_0Shelf (default: TRUE)
 *	Specifies if a "shelf" should be drawn parallel to the x-axis where y
 *	is 0.  It is only applicable for 3D charts and if there are y values
 *	less than and greater than 0.
 *
 * gdccht_grid (default: TRUE)
 *	Specifies if the background grid should be displayed.  The x and y-axis
 *	tick marks are part of the grid.
 *
 * gdccht_GridColor (default: grey)
 *	The color of the background grid.
 *
 * gdccht_border (default: TRUE)
 *	Specifies if a border line (using gdc_LineColor) is displayed around
 *	the chart.
 *
 * gdccht_xaxis (default: TRUE)
 *	Specifies if the x-axis is displayed.  The x-axis only consists of the
 *	labels.
 *
 * gdccht_yaxis (default: TRUE)
 *	Specifies if the y-axis is displayed.  The y-axis only consists of the
 *	y-axis values.
 *
 * gdccht_yaxis2 (default: TRUE)
 *	Specifies if the right hand y-axis is displayed.  The right hand y-axis
 *	only consists of the right hand y-axis values.  It is only applicable
 *	to *COMBO* chart types.
 *
 * gdccht_xaxisfont_size (default: gdc_label_size)
 *	The font to use for the x-axis's labels.
 *
 * gdccht_yaxisfont_size (default: gdc_label_size)
 *	The font to use for the y-axis's values.
 *
 * gdccht_xtitle (default: NULL)
 *	The text of the chart's optional x-axis title.
 *
 * gdccht_ytitle (default: NULL)
 *	The text of the chart's optional y-axis title.
 *
 * gdccht_ytitle2 (default: NULL)
 *	The text of the chart's optional right hand y-axis title.  It is only
 *	applicable to *COMBO* chart types.
 *
 * gdccht_xtitle_size (default: medium bold)
 *	The font to use for the chart's x-axis title.
 *
 * gdccht_ytitle_size (default: medium bold)
 *	The font to use for the chart's y-axis titles.
 *
 * gdccht_ylabel_fmt (default: display in fractions)
 *	The printf() format string to use to format the y-axis values.
 *
 * gdccht_ylabel2_fmt (default: "%.0f")
 *	The printf() format string to use to format the right hand y-axis
 *	values.  It is only applicable to *COMBO* chart types.
 *
 * gdccht_XTitleColor (default: the default color)
 *	The color to use for the x-axis title.
 *
 * gdccht_YTitleColor (default: the default color)
 *	The color to use for the y-axis title.
 *
 * gdccht_YTitle2Color (default: the default color)
 *	The color to use for the right hand y-axis title.  It is only
 *	applicable to *COMBO* chart types.
 *
 * gdccht_XLabelColor (default: gdc_LabelColor)
 *	The color to use for x-axis labels.
 *
 * gdccht_YLabelColor (default: gdc_LabelColor)
 *	The color to use for y-axis labels.
 *
 * gdccht_YLabel2Color (default: gdc_LabelColor)
 *	The color to use for right hand y-axis labels.  It is only applicable
 *	to *COMBO* chart types.
 *
 * gdccht_annotation (default: NULL)
 *	A pointer to a structure describing the annotation to add to the chart.
 *
 * gdccht_annotation_font (default: small)
 *	The font to use for the annotation.
 *
 * gdccht_xlabel_density (default: 80)
 *	Specify, as a %age, the amount of horizontal space allocated to a
 *	x-axis label that is taken up by the label itself.  As this figure
 *	increases, the gap between consecutive labels decreases.  If the gap
 *	becomes too small, then some labels are not displayed.  If the
 *	specified value is less than 0 then all labels will be displayed, even
 *	if they overlap.
 *
 * gdccht_ylabel_density (default: 80)
 *	Specify, as a %age, the amount of vertical space allocated to a y-axis
 *	value that is taken up by the value itself.  As this figure increases,
 *	the gap between consecutive values decreases.  If the gap becomes too
 *	small, then the interval between values is increased.
 *
 * gdccht_requested_ymin (default: DBL_MIN)
 *	The value of the lower bound of the y-axis (if it is less than the
 *	smallest data point).
 *
 * gdccht_requested_ymax (default: DBL_MIN)
 *	The value of the upper bound of the y-axis (if it is more than the
 *	largest data point).
 *
 * gdccht_requested_yinterval (default: DBL_MIN)
 *	The difference between consecutive y-axis values.  The default is to
 *	use the smallest interval possible while maintaining the requested
 *	y-axis label density (specified by gdccht_ylabel_density).
 *
 * gdccht_HLC_style (default: GDC_HLC_HORIZ_CONNECTED)
 *	Specify how to mark the close point on HLC charts.  GDC_HLC_DIAMOND
 *	means draw a diamond, GDC_HLC_HORIZ_CONNECTED means draw a horizontal
 *	line between the close point and the high/low line to the right,
 *	GDC_HLC_CONNECTED means draw a line between the close point and the
 *	close point to the right.
 *
 * gdccht_HLC_cap_width (default: 25)
 *	The width, as a %age of the x-axis column, of the horizontal bar
 *	displayed at the high and low points on HLC charts.  A value of zero
 *	means the bar's are not drawn.  The value also determines the size of
 *	any diamon drawn at the close point.
 *
 * gdccht_num_scatter_pts (default: 0)
 *	The number of scatter points to display (ie. the number of elements in
 *	the gdccht_scatter array).
 *
 * gdccht_scatter (default: NULL)
 *	The array of scatter pointers.
 */


/*
 * Set the defaults for the attribute values.
 */

void GDC_set_defaults(GDC_AttributeValues *av)
{
	static GDC_AttributeValues defaults = {
		TRUE,			/* gdc_generate_image */
		FALSE,			/* gdc_hold_image */
		NULL,			/* gdc_image */
		0L,			/* gdc_BGColor */
		TRUE,			/* gdc_transparent_bg */
		NULL,			/* gdc_title */
		GDC_MEDBOLD,		/* gdc_title_size */
		GDC_DFLTCOLOR,		/* gdc_TitleColor */
		GDC_SMALL,		/* gdc_label_size */
		GDC_DFLTCOLOR,		/* gdc_LabelColor */
		GDC_DFLTCOLOR,		/* gdc_LineColor */
		45,			/* gdc_3d_angle */
		5,			/* gdc_3d_depth */

		GDC_NOCOLOR,		/* gdcpie_EdgeColor */
		100,			/* gdcpie_label_dist */
		FALSE,			/* gdcpie_label_line */
		GDCPIE_PCT_NONE,	/* gdcpie_percent_labels */
		-1,			/* gdcpie_other_threshold */

		GDC_STACK_BEHIND,	/* gdccht_stack_type */
		75,			/* gdccht_bar_width */
		TRUE,			/* gdccht_0Shelf */
		TRUE,			/* gdccht_grid */
		0xa0a0a0L,		/* gdccht_GridColor */
		TRUE,			/* gdccht_border */
		TRUE,			/* gdccht_xaxis */
		TRUE,			/* gdccht_yaxis */
		TRUE,			/* gdccht_yaxis2 */
		GDC_DEFAULT_FONT,	/* gdccht_xaxisfont_size */
		GDC_DEFAULT_FONT,	/* gdccht_yaxisfont_size */
		NULL,			/* gdccht_xtitle */
		NULL,			/* gdccht_ytitle */
		NULL,			/* gdccht_ytitle2 */
		GDC_MEDBOLD,		/* gdccht_xtitle_size */
		GDC_MEDBOLD,		/* gdccht_ytitle_size */
		NULL,			/* gdccht_ylabel_fmt */
		NULL,			/* gdccht_ylabel2_fmt */
		GDC_DFLTCOLOR,		/* gdccht_XTitleColor */
		GDC_DFLTCOLOR,		/* gdccht_YTitleColor */
		GDC_DFLTCOLOR,		/* gdccht_YTitle2Color */
		GDC_DFLTCOLOR,		/* gdccht_XLabelColor */
		GDC_DFLTCOLOR,		/* gdccht_YLabelColor */
		GDC_DFLTCOLOR,		/* gdccht_YLabel2Color */
		NULL,			/* gdccht_annotation */
		GDC_SMALL,		/* gdccht_annotation_font */
		80,			/* gdccht_xlabel_density */
		80,			/* gdccht_ylabel_density */
		DBL_MIN,		/* gdccht_requested_ymin */
		DBL_MIN,		/* gdccht_requested_ymax */
		DBL_MIN,		/* gdccht_requested_yinterval */
		GDC_HLC_HORIZ_CONNECTED,/* gdccht_HLC_style */
		25,			/* gdccht_HLC_cap_width */
		0,			/* gdccht_num_scatter_pts */
		NULL,			/* gdccht_scatter */
	};

	*av = defaults;
}
