/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module implements the Python part of the GraphPoint class.
 */


#include <stddef.h>
#include <float.h>

#include "pymodint.h"


static int graphPointCtor(pymodPyBase *,PyObject *,PyObject *);
static int graphPointValueConvFrom(PyObject *,void *);
static PyObject *graphPointValueConvTo(void *);


/*****************************************************************************/
/* Describe the GraphPoint class.                                            */
/*****************************************************************************/

/* The wrapped toolkit object. */

static PyTypeObject graphPointType = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,
	"__vpGraphPoint__",
	sizeof (pymodPyGraphPoint),
	0,
	pymodDeallocGraphPoint,
};

/* The properties of this class. */

propDef pymodPropGraphPointColor = {
	"color",
	instanceProp,
	offsetof(pymodPyGraphPoint,graphPoint.color),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
	NULL,
	&pymodClassColor,
};

propDef pymodPropGraphPointValue = {
	"value",
	bespokeProp,
	offsetof(pymodPyGraphPoint,graphPoint.value),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
	NULL,
	NULL,
	graphPointValueConvFrom,
	graphPointValueConvTo,
};

/*
 * The value property is first so that we can create GraphPoint instances on
 * the fly with a value.
 */

static classPropDef graphPointProperties[] = {
	{&pymodPropGraphPointValue, pymodNullHandler, },
	{&pymodPropGraphPointColor, pymodNullHandler, },
	{NULL}
};

/* The class definition. */
 
classDef pymodClassGraphPoint = {
	"GraphPoint",
	graphPointProperties,
	NULL,
	NULL,
	NULL,
	graphPointCtor,
	&graphPointType,
};


/*
 * The class ctor.
 */

static int graphPointCtor(pymodPyBase *self,PyObject *args,PyObject *kwds)
{
	if (pymodParseArgs(&pymodClassGraphPoint,self,args,kwds,graphPointProperties,"c") < 0)
		return -1;

	/* Nothing else to do. */

	return 0;
}


/*
 * Convert a value property value from Python.
 */

static int graphPointValueConvFrom(PyObject *val,void *p)
{
	pymodBespokeGraphPointValue *cpv = (pymodBespokeGraphPointValue *)p;

	cpv -> simple = cpv -> high = cpv -> low = cpv -> close = DBL_MIN;

	if (val == NULL || val == Py_None)
		return 0;

	if (PyTuple_Check(val))
	{
		if (PyTuple_GET_SIZE(val) != 3)
		{
			PyErr_Format(PyExc_TypeError,"Tuple of HLC values must have 3 elements");
			return -1;
		}

		if (pymodConvertDouble(PyTuple_GET_ITEM(val,0),&cpv -> high) < 0)
			return -1;

		if (pymodConvertDouble(PyTuple_GET_ITEM(val,1),&cpv -> low) < 0)
			return -1;

		if (pymodConvertDouble(PyTuple_GET_ITEM(val,2),&cpv -> close) < 0)
			return -1;

		return 0;
	}

	return pymodConvertDouble(val,&cpv -> simple);
}


/*
 * Convert a value property value to Python.
 */

static PyObject *graphPointValueConvTo(void *p)
{
	pymodBespokeGraphPointValue *cpv = (pymodBespokeGraphPointValue *)p;

	if (cpv -> simple != DBL_MIN)
		return PyFloat_FromDouble(cpv -> simple);

	if (cpv -> high != DBL_MIN || cpv -> low != DBL_MIN || cpv -> close != DBL_MIN)
	{
		PyObject *tup, *val;

		if ((tup = PyTuple_New(3)) == NULL)
			return NULL;

		if (cpv -> high == DBL_MIN)
		{
			Py_INCREF(Py_None);
			val = Py_None;
		}
		else if ((val = PyFloat_FromDouble(cpv -> high)) == NULL)
			return NULL;

		PyTuple_SET_ITEM(tup,0,val);

		if (cpv -> low == DBL_MIN)
		{
			Py_INCREF(Py_None);
			val = Py_None;
		}
		else if ((val = PyFloat_FromDouble(cpv -> low)) == NULL)
			return NULL;

		PyTuple_SET_ITEM(tup,1,val);

		if (cpv -> close == DBL_MIN)
		{
			Py_INCREF(Py_None);
			val = Py_None;
		}
		else if ((val = PyFloat_FromDouble(cpv -> close)) == NULL)
			return NULL;

		PyTuple_SET_ITEM(tup,2,val);

		return tup;
	}

	Py_INCREF(Py_None);
	return Py_None;
}
