/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module implements the Python part of the PieChart class.
 */


#include <stddef.h>

#include "gdchart.h"
#include "pymodint.h"


static int pieChartCtor(pymodPyBase *,PyObject *,PyObject *);
static PyObject *pieChartPlot(pymodPyBase *,PyObject *);


/*****************************************************************************/
/* Describe the PieChart class.                                              */
/*****************************************************************************/

/* The wrapped toolkit object. */

static PyTypeObject pieChartType = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,
	"__vpPieChart__",
	sizeof (pymodPyPieChart),
	0,
	pymodDeallocPieChart,
};

/* The methods of this class. */

static PyMethodDef pieChartMethods[] = {
	{"plot", (PyCFunction)pieChartPlot, METH_VARARGS, },
	{NULL}
};

/* The properties of this class. */

propDef pymodPropPieChartOutlineColor = {
	"outlineColor",
	instanceProp,
	offsetof(pymodPyPieChart,pieChart.outlineColor),
	0,
	{PROP_VALID_NONE_OK, },
	{PROP_VALUE_IS_DEFAULT|PROP_VALUE_IS_NONE, },
	NULL,
	&pymodClassColor,
};

propDef pymodPropPieChartSeries = {
	"series",
	instanceProp,
	offsetof(pymodPyPieChart,pieChart.series),
	PROP_FLAG_LIST,
	{PROP_VALID_EMPTY_OK, },
	{0, },
	NULL,
	&pymodClassPieSlice,
};

static classPropDef pieChartProperties[] = {
	{&pymodPropControlParent, },
	{&pymodPropChartHeight, pymodNullHandler, },
	{&pymodPropChartWidth, pymodNullHandler, },
	{&pymodPropControlColumn, },
	{&pymodPropChartLabelColor, pymodNullHandler, },
	{&pymodPropPieChartOutlineColor, pymodNullHandler, },
	{&pymodPropControlRow, },
	{&pymodPropPieChartSeries, pymodNullHandler, },
	{&pymodPropChartTitle, pymodNullHandler, },
	{&pymodPropChartTitleColor, pymodNullHandler, },
	{NULL}
};


/* The class definition. */
 
classDef pymodClassPieChart = {
	"PieChart",
	pieChartProperties,
	pieChartMethods,
	NULL,
	NULL,
	pieChartCtor,
	&pieChartType,
};


/*
 * The class ctor.
 */

static int pieChartCtor(pymodPyBase *self,PyObject *args,PyObject *kwds)
{
	pymodDataPieChart *pcd = &((pymodPyPieChart *)self) -> pieChart;

	if (pymodParseArgs(&pymodClassPieChart,self,args,kwds,pieChartProperties,"c") < 0)
		return -1;

	if (pymodInitControl1(self) < 0)
		return -1;

	/*
	 * Call the driver.
	 *
	 * parent is the parent toolkit object.
	 */

	if ((pcd -> chart.control.base.tkPtr = (*((pymodPieChartDrivers *)pymodClassPieChart.drivers) -> ctor)(
			(PyObject *)self,
			pcd -> chart.control.parent -> container.control.base.tkPtr,
			pcd -> chart.width,
			pcd -> chart.height)) == NULL)
	{
		pymodBadToolkitCtor(pymodClassPieChart.name);
		return -1;
	}

	pymodInitControl2(self);

	return 0;
}


/*
 * The method plot.
 */

static PyObject *pieChartPlot(pymodPyBase *self,PyObject *args)
{
	int image_size, i, nrslices;
	void *image;
	pymodPyPieChart *pcw;
	PyObject *slist;
	GDCPIE_DataPoint *slices, *sp;
	GDC_AttributeValues av;

	if (pymodParseArgs(&pymodClassPieChart,self,args,NULL,NULL,
				"mplot",
				&pcw) < 0)
		return NULL;

	/* Check we have something to plot. */

	if ((slist = pcw -> pieChart.series) == NULL || (nrslices = PyList_GET_SIZE(slist)) == 0)
	{
		PyErr_Format(PyExc_ValueError,"There is no data to plot");
		return NULL;
	}

	/* Convert the data. */

	if ((slices = Py_Malloc(sizeof (GDCPIE_DataPoint) * nrslices)) == NULL)
		return NULL;

	sp = slices;

	for (i = 0; i < nrslices; ++i)
	{
		pymodPyPieSlice *psw;
		pymodDataPieSlice *psd;

		psw = (pymodPyPieSlice *)PyList_GET_ITEM(slist,i);
		psd = &psw -> pieSlice;

		sp -> value = psd -> value;
		sp -> color = pymodChartColor(psd -> color);
		sp -> offset = psd -> offset;

		sp -> label = (psd -> label != NULL) ? PyString_AS_STRING(psd -> label) : NULL;

		++sp;
	}

	/* Set the other properties. */

	GDC_set_defaults(&av);

	av.gdc_BGColor = pymodChartColor(pcw -> pieChart.chart.control.background);

	av.gdc_LabelColor = pymodChartColor(pcw -> pieChart.chart.labelColor);

	if (pcw -> pieChart.outlineColor != NULL)
		av.gdcpie_EdgeColor = pymodChartColor(pcw -> pieChart.outlineColor);

	if (pcw -> pieChart.chart.title != NULL)
		av.gdc_title = PyString_AS_STRING(pcw -> pieChart.chart.title);

	av.gdc_TitleColor = pymodChartColor(pcw -> pieChart.chart.titleColor);

	/* Generate the image. */

	image = GDC_out_pie(pcw -> pieChart.chart.width,
			    pcw -> pieChart.chart.height,
			    &image_size,GDC_2DPIE,&av,nrslices,slices);

	Py_Free((ANY *)slices);

	if (image == NULL)
	{
		PyErr_Format(PyExc_SystemError,"Internal error: unable to create pie chart");
		return NULL;
	}

	/* Call the driver. */

	(*((pymodPieChartDrivers *)pymodClassPieChart.drivers) -> plot)(
		pcw -> pieChart.chart.control.base.tkPtr,
		image,
		image_size);

	Py_Free((ANY *)image);

	Py_INCREF(Py_None);
	return Py_None;
}
