/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module implements properties.
 */


#include <stdarg.h>
#include <string.h>

#include "pymodint.h"


static classPropDef *findProp(pymodPyBase *,PyObject *);
static int checkGivenProperties(PyObject *,classPropDef *,int,int);
static int propertyToCValue(classPropDef *,propValidDef *,propValueDef *,PyObject *,pymodPyBase *,va_list *);
static int convertEnumList(propDef *,propValidDef *,propValueDef *,PyObject *);
static int convertEnum(propDef *,propValidDef *,propValueDef *,PyObject *,int *);
static PyObject *convertStringList(propDef *,propValidDef *,propValueDef *,PyObject *);
static int convertString(propDef *,propValidDef *,propValueDef *,PyObject *,PyObject **);
static PyObject *convertInstanceList(propDef *,propValidDef *,PyObject *);
static int convertInstance(propDef *,propValidDef *,PyObject *,PyObject **);
static void getValidityAndValue(classPropDef *,propValidDef **,propValueDef **);
static void badType(propDef *);
static void propDesc(propDef *,char **,char **);
static void deleteProp(propDef *,void *);
static PyObject *checkMakeInstance(classDef *,PyObject *);
static int checkCanBeNone(propDef *,propValidDef *);
static int checkList(propDef *,propValidDef *,PyObject *);
static int isInstance(PyObject *,classDef *);


/*
 * Implement the base __getattr__ function to handle the getting of property
 * values.
 */

PyObject *pymodCallGetattr(PyObject *ignore,PyObject *args)
{
	PyObject *inst, *attrobj, *val;
	pymodPyBase *wo;
	classPropDef *cpd;
	propValueDef *pdd;
	void *ptr;

	/* Get the instance and attribute name. */

	if (!PyArg_ParseTuple(args,"O!S",&PyInstance_Type,&inst,&attrobj))
	{
		PyErr_Format(PyExc_SystemError,"Internal error: bad arguments to __vpGetattr__");
		return NULL;
	}

	/* Get the wrapped object. */

	if ((wo = pymodGetWrappedObject(inst)) == NULL)
		return NULL;

	/* Get the property. */

	if ((cpd = findProp(wo,attrobj)) == NULL)
	{
		PyErr_SetObject(PyExc_AttributeError,attrobj);
		return NULL;
	}

	/* Get the address of the property value. */

	ptr = (void *)wo + cpd -> prop -> offset;

	switch (cpd -> prop -> type)
	{
	case intProp:
	case uintProp:
		val = PyInt_FromLong(*(long *)ptr);
		break;

	case doubleProp:
		val = PyFloat_FromDouble(*(double *)ptr);
		break;

	case pyobjectProp:
		if ((val = *(PyObject **)ptr) == NULL)
			val = Py_None;

		Py_INCREF(val);

		break;

	case stringProp:
		if (propIsList(cpd -> prop))
		{
			int i;
			PyObject *list;

			if ((list = *(PyObject **)ptr) == NULL)
				val = PyList_New(0);
			else if ((val = PyList_New(PyList_GET_SIZE(list))) == NULL)
				return NULL;

			for (i = 0; i < PyList_GET_SIZE(list); ++i)
			{
				PyObject *itm = PyList_GET_ITEM(list,i);

				if (itm == NULL)
					itm = Py_None;

				PyList_SET_ITEM(val,i,itm);
				Py_INCREF(itm);
			}
		}
		else
		{
			if ((val = *(PyObject **)ptr) == NULL)
				val = Py_None;

			Py_INCREF(val);
		}

		break;

	case enumProp:
		if (propIsList(cpd -> prop))
		{
			int enumval;

			if ((enumval = *(int *)ptr) == 0)
				val = PyList_New(0);
			else
			{
				int i, len, mask;

				/* Count the length of list needed. */

				len = 0;

				for (mask = 1; mask != 0; mask <<= 1)
					if (enumval & mask)
						++len;

				if ((val = PyList_New(len)) == NULL)
					return NULL;

				/* Add the values to the list. */

				i = 0;

				for (mask = 1; mask != 0; mask <<= 1)
					if (enumval & mask)
					{
						PyObject *itm;

						if ((itm = pymodConvertEnum(cpd -> prop -> ed,enumval & mask)) == NULL)
							return NULL;

						PyList_SET_ITEM(val,i,itm);

						++i;
					}
			}
		}
		else
		{
			int enumval;

			if ((enumval = *(int *)ptr) == 0)
			{
				Py_INCREF(Py_None);
				val = Py_None;
			}
			else if ((val = pymodConvertEnum(cpd -> prop -> ed,enumval)) == NULL)
				return NULL;
		}

		break;

	case bespokeProp:
		val = (*cpd -> prop -> convTo)(ptr);
		break;

	case instanceProp:
		if (propIsList(cpd -> prop))
		{
			int i;
			PyObject *list;

			if ((list = *(PyObject **)ptr) == NULL)
				val = PyList_New(0);
			else if ((val = PyList_New(PyList_GET_SIZE(list))) == NULL)
				return NULL;

			for (i = 0; i < PyList_GET_SIZE(list); ++i)
			{
				PyObject *itm = PyList_GET_ITEM(list,i);

				if (itm == NULL)
					itm = Py_None;
				else
					itm = ((pymodPyBase *)itm) -> base.instance;

				PyList_SET_ITEM(val,i,itm);
				Py_INCREF(itm);
			}
		}
		else
		{
			pymodPyBase *bw = *(pymodPyBase **)ptr;

			if (bw == NULL)
				val = Py_None;
			else
				val = bw -> base.instance;

			Py_INCREF(val);
		}

		break;
	}

	return val;
}


/*
 * Implement the base __setattr__ function to handle the setting of property
 * values.
 */

PyObject *pymodCallSetattr(PyObject *ignore,PyObject *args)
{
	PyObject *inst, *attrobj, *valobj;
	pymodPyBase *wo;
	classPropDef *cpd;

	/* Get the instance, attribute name and value. */

	if (!PyArg_ParseTuple(args,"O!SO",&PyInstance_Type,&inst,&attrobj,&valobj))
	{
		PyErr_Format(PyExc_SystemError,"Internal error: bad arguments to __vpSetattr__");
		return NULL;
	}

	/* Get the wrapped object. */

	if ((wo = pymodGetWrappedObject(inst)) == NULL)
		return NULL;

	/* Get the property. */

	if ((cpd = findProp(wo,attrobj)) == NULL)
	{
		/*
		 * It isn't a property so just save the value in the instance
		 * dictionary.
		 */

		if (PyDict_SetItem(((PyInstanceObject *)inst) -> in_dict,attrobj,valobj) < 0)
			return NULL;
	}
	else if (cpd -> setprop == NULL)
	{
		/* It can't be changed on the fly. */

		PyErr_Format(PyExc_TypeError,"The %s property can only be set when the instance is created",cpd -> prop -> name);

		return NULL;
	}
	else
	{
		propValidDef *pvd;
		propValueDef *pdd;

		getValidityAndValue(cpd,&pvd,&pdd);

		if (propertyToCValue(cpd,pvd,pdd,valobj,wo,NULL) < 0)
			return NULL;

		/* Call the handler. */

		if ((*cpd -> setprop)(wo) < 0)
		{
			PyErr_Format(PyExc_SystemError,"Internal error: the toolkit property handler returned an error");

			return NULL;
		}
	}

	Py_INCREF(Py_None);
	return Py_None;
}


/*
 * A property handler for those properties that only need to set the value and
 * not call the toolkit.
 */

int pymodNullHandler(pymodPyBase *ignore)
{
	return 0;
}


/*
 * Look for a property of an instance.  Return the property meta-data.
 */

static classPropDef *findProp(pymodPyBase *wo,PyObject *attr)
{
	classPropDef *cpd;

	if ((cpd = wo -> base.cd -> props) != NULL)
		while (cpd -> prop != NULL)
		{
			if (strcmp(cpd -> prop -> name,PyString_AS_STRING(attr)) == 0)
				return cpd;

			++cpd;
		}

	/* Nothing found. */

	return NULL;
}


/*
 * Parse the arguments for a class ctor, a class method or an unbound class
 * function (ie. a static method) from positional arguments and/or a dictionary
 * of keywords.  Return -1 is there was an error.
 *
 * On Entry:
 *	cd	is a pointer to the class meta-data
 *	self	is the Python object that represents the instance of the class.
 *		If this is a method and it is NULL then the method was called
 *		as class.method(self,) rather than self.method() and the
 *		instance is expected to be found as the first element in the
 *		args tuple.  If this is a static method and it is not NULL then
 *		an exception is raised to force the user to call it as
 *		class.method().
 *	args	is the tuple of positional arguments.
 *	kwds	is the dictionary of keyword arguments.  It may be NULL if this
 *		is a ctor, and must be NULL otherwise.
 *	props	is an array of property definitions describing the valid
 *		arguments.
 *	mname	is a string.  The first character is 'c', 'm' or 's' depending
 *		on whether this is a ctor, a method or a static method
 *		respectively.  Both 'm' and 's' are immediately followed by the
 *		name of the method which is used for error reporting.
 *	...	if this is a ctor then there are no more parameters and the
 *		parsed values are written to the self.  If this is a method
 *		then the next parameter is a pointer to a variable to which a
 *		pointer to the wrapped object is written.  If this is a method
 *		or a static method then all the remaining parameters are
 *		pointers to valiables, one for each argument, into which the
 *		parsed values are written.
 */

int pymodParseArgs(classDef *cd,pymodPyBase *self,PyObject *args,PyObject *kwds,classPropDef *props,char *mname,...)
{
	int nrargs, propnr, inCtor, argnr;
	pymodPyBase *ptrs;
	va_list va;

	/* Assume we will be returning values through the argument list. */

	ptrs = NULL;

	va_start(va,mname);

	/* Get the number of positional arguments. */

	if ((nrargs = PyTuple_Size(args)) < 0 || (kwds != NULL && !PyDict_Check(kwds)))
	{
		PyErr_Format(PyExc_SystemError,"Internal error: trying to parse invalid argument types");

		return -1;
	}

	/* Handle self. */

	argnr = 0;
	inCtor = 0;

	switch (*mname++)
	{
	case 's':
		{
			/* A static class method. */

			if (self != NULL)
			{
				PyErr_Format(PyExc_RuntimeError,"%s.%s() is a function and cannot be called as self.%s()",cd -> name,mname,mname);
				return -1;
			}

			break;
		}

	case 'm':
		{
			/* A class method. */

			if (self == NULL)
			{
				/*
				 * Called as Class.Method(self,...) so get self
				 * from the argument list.
				 */

				if (nrargs == 0 || !isInstance(PyTuple_GET_ITEM(args,0),cd))
				{
					PyErr_Format(PyExc_RuntimeError,"%s.%s() must be called with an instance of class %s as its first argument",cd -> name,mname,cd -> name);
					return -1;
				}

				++argnr;
				--nrargs;

				/* Get the wrapped object. */

				if ((self = pymodGetWrappedObject(PyTuple_GET_ITEM(args,0))) == NULL)
					return -1;
			}

			/*
			 * Check the toolkit object is still there in case we
			 * are keeping references around for dead widgets.
			 */

			if (self -> base.tkPtr == NULL)
			{
				PyErr_Format(PyExc_RuntimeError,"The underlying toolkit object has been deleted");
				return -1;
			}

			/* Return the wrapped object. */

			*va_arg(va,pymodPyBase **) = self;

			break;
		}

	case 'c':
		/* A class ctor. */

		ptrs = self;
		mname = NULL;
		inCtor = 1;
		break;

	default:
		PyErr_Format(PyExc_SystemError,"Internal error: invalid format string");
	}

	/* Check any given named properties are valid. */

	if (kwds != NULL && checkGivenProperties(kwds,props,nrargs,inCtor) < 0)
		return -1;

	/* Convert the positional arguments. */

	propnr = 0;

	while (props != NULL && props[propnr].prop != NULL)
	{
		PyObject *arg;
		propValidDef *pvd;
		propValueDef *pdd;
		classPropDef *cpd = &props[propnr];

		/* Skip it if we are in a ctor and this isn't allowed. */

		if (inCtor && propNotInCtor(cpd -> prop))
		{
			++propnr;
			continue;
		}

		/* Get the validity information and any default value. */

		getValidityAndValue(cpd,&pvd,&pdd);

		/*
		 * Get the object to convert from the positional arguments if
		 * there are any left, otherwise from the keyword dictionary.
		 */

		if (argnr < nrargs)
		{
			arg = PyTuple_GET_ITEM(args,argnr);
			++argnr;
		}
		else if (kwds != NULL)
			arg = PyDict_GetItem(kwds,cpd -> prop -> nameobj);
		else
			arg = NULL;

		/*
		 * If we don't have a value then we must have a default.  The
		 * default for a list is always an empty list - and whether or
		 * not an empty list is allowed is checked later.
		 */

		if (arg == NULL && !propIsList(cpd -> prop) && !propValueIsDefault(pdd))
		{
			PyErr_Format(PyExc_TypeError,"The %s property must be given",cpd -> prop -> name);

			return -1;
		}

		/* Convert the value. */

		if (propertyToCValue(cpd,pvd,pdd,arg,ptrs,&va) < 0)
			return -1;

		++propnr;
	}

	va_end(va);

	/* Check that there weren't too many arguments. */

	if (nrargs > propnr)
	{
		if (mname != NULL)
			PyErr_Format(PyExc_TypeError,"Too many arguments given to %s.%s()",cd -> name,mname);
		else
			PyErr_Format(PyExc_TypeError,"Too many arguments given to %s()",cd -> name);

		return -1;
	}

	return 0;
}


/*
 * Get the validity information and any default value for a property.
 */

static void getValidityAndValue(classPropDef *cpd,propValidDef **pvdp,propValueDef **pddp)
{
	propValidDef *pvd;
	propValueDef *pdd;

	/* Just check if the default defaults have been overridden. */

	pvd = &cpd -> defvalid;

	if (!propValidOverride(pvd))
		pvd = &cpd -> prop -> defdefvalid;

	pdd = &cpd -> defvalue;

	if (!propValueOverride(pdd))
		pdd = &cpd -> prop -> defdefvalue;

	*pvdp = pvd;
	*pddp = pdd;
}


/*
 * Check that all keywords are valid property names and that they haven't
 * already been given as positional arguments.  Return -1 if there was an
 * error.
 */

static int checkGivenProperties(PyObject *kwds,classPropDef *props,int nrargs,int inCtor)
{
	int pos, propnr;
	PyObject *key, *value;

	pos = 0;

	while (PyDict_Next(kwds,&pos,&key,&value))
	{
		char *propname;

		/* Find which position this would be. */

		if (!PyString_Check(key))
		{
			PyErr_Format(PyExc_SystemError,"Internal error: non-string key in keyword dictionary");

			return -1;
		}

		propname = PyString_AS_STRING(key);

		propnr = -1;

		if (props != NULL)
		{
			int i;

			for (i = 0; props[i].prop != NULL; ++i)
				if (strcmp(props[i].prop -> name,propname) == 0)
				{
					if (inCtor && propNotInCtor(props[i].prop))
					{
						PyErr_Format(PyExc_TypeError,"%s cannot be set when creating an instance",propname);

						return -1;
					}

					propnr = i;
					break;
				}
		}

		if (propnr < 0)
		{
			PyErr_Format(PyExc_TypeError,"%s is an unknown property in this context",propname);

			return -1;
		}

		if (propnr < nrargs)
		{
			PyErr_Format(PyExc_TypeError,"%s has aleady given as a positional argument",propname);

			return -1;
		}
	}

	return 0;
}


/*
 * Convert a property to a C value.  Return zero if there was no error.
 */

static int propertyToCValue(classPropDef *cpd,propValidDef *pvd,propValueDef *pdd,PyObject *arg,pymodPyBase *ptrs,va_list *vap)
{
	void *ptr;

	/* Get the address of the destination. */

	if (ptrs != NULL)
	{
		ptr = (void *)ptrs + cpd -> prop -> offset;

		/* Clear any previous value from the instance record. */

		deleteProp(cpd -> prop,ptr);
	}
	else
		ptr = va_arg(*vap,void *);

	switch (cpd -> prop -> type)
	{
	case intProp:
	case uintProp:
		{
			long *p = (long *)ptr;

			if (arg == NULL)
				*p = pdd -> defint;
			else
			{
				PyErr_Clear();

				*p = PyInt_AsLong(arg);

				if (PyErr_Occurred() != NULL)
				{
					badType(cpd -> prop);
					return -1;
				}

				if (cpd -> prop -> type == uintProp && *p < 0)
				{
					badType(cpd -> prop);
					return -1;
				}
			}

			break;
		}

	case doubleProp:
		{
			double *p = (double *)ptr;

			if (pymodConvertDouble(arg,p) < 0)
			{
				badType(cpd -> prop);
				return -1;
			}

			break;
		}

	case stringProp:
		{
			PyObject **p = (PyObject **)ptr;

			if (propIsList(cpd -> prop))
			{
				if ((*p = convertStringList(cpd -> prop,pvd,pdd,arg)) == NULL)
					return -1;
			}
			else if (convertString(cpd -> prop,pvd,pdd,arg,p) < 0)
				return -1;

			break;
		}

	case enumProp:
		{
			int *p = (int *)ptr;

			if (propIsList(cpd -> prop))
			{
				if ((*p = convertEnumList(cpd -> prop,pvd,pdd,arg)) < 0)
					return -1;
			}
			else if (convertEnum(cpd -> prop,pvd,pdd,arg,p) < 0)
				return -1;

			break;
		}

	case pyobjectProp:
		{
			PyObject **p = (PyObject **)ptr;

			if (arg == NULL)
				arg = Py_None;

			Py_INCREF(arg);

			*p = arg;

			break;
		}

	case bespokeProp:
		if ((*cpd -> prop -> convFrom)(arg,ptr) < 0)
			return -1;

		break;

	case instanceProp:
		{
			PyObject **p = (PyObject **)ptr;

			if (propIsList(cpd -> prop))
			{
				if ((*p = convertInstanceList(cpd -> prop,pvd,arg)) == NULL)
					return -1;
			}
			else if (convertInstance(cpd -> prop,pvd,arg,p) < 0)
				return -1;

			break;
		}
	}

	return 0;
}


/*
 * Check if a property is allowed to be None.
 */

static int checkCanBeNone(propDef *pd,propValidDef *pvd)
{
	if (!propValidCanBeNone(pvd))
	{
		if (propIsList(pd))
			PyErr_Format(PyExc_TypeError,"The %s property cannot have elements that are None",pd -> name);
		else
			PyErr_Format(PyExc_TypeError,"The %s property cannot be None",pd -> name);

		return -1;
	}

	return 0;
}


/*
 * Convert a property to a string.  Return -1 if there was an error.
 */

static int convertString(propDef *pd,propValidDef *pvd,propValueDef *pdd,PyObject *arg,PyObject **p)
{
	if (arg == NULL)
	{
		/* Set the default value - there will be one. */

		if (!propValueIsNone(pdd) && (*p = PyString_FromString(pdd -> defstr)) == NULL)
			return -1;
	}
	else if (arg == Py_None)
	{
		if (checkCanBeNone(pd,pvd))
			return -1;

		arg = NULL;
	}
	else if (PyString_Check(arg))
		Py_INCREF(arg);
	else
	{
		badType(pd);
		return -1;
	}

	*p = arg;

	return 0;
}


/*
 * Convert a property to a list of string.  Return NULL if there was an error.
 */

static PyObject *convertStringList(propDef *pd,propValidDef *pvd,propValueDef *pdd,PyObject *arg)
{
	int i;
	PyObject *new;

	/* Check it is a valid list. */

	if (checkList(pd,pvd,arg) < 0)
		return NULL;

	/* Copy the list while converting each element. */

	if ((new = PyList_New(PyList_GET_SIZE(arg))) == NULL)
		return NULL;

	for (i = 0; i < PyList_GET_SIZE(arg); ++i)
	{
		PyObject *itm = PyList_GET_ITEM(arg,i), *newitm;

		if (convertString(pd,pvd,pdd,itm,&newitm) < 0)
		{
			Py_DECREF(new);

			return NULL;
		}

		PyList_SET_ITEM(new,i,newitm);
	}

	return new;
}


/*
 * Convert a property to an enum.  Return -1 if there was an error.
 */

static int convertEnum(propDef *pd,propValidDef *pvd,propValueDef *pdd,PyObject *arg,int *p)
{
	int enumval;

	if (arg == NULL)
	{
		/* Set the default value - there will be one. */

		if (propValueIsNone(pdd))
			enumval = 0;
		else
			enumval = pdd -> defenum;
	}
	else if (arg == Py_None)
	{
		if (checkCanBeNone(pd,pvd))
			return -1;

		enumval = 0;
	}
	else if (arg -> ob_type == pd -> ed -> type)
		enumval = ((enumObject *)arg) -> evd -> intval;
	else
	{
		badType(pd);
		return -1;
	}

	*p = enumval;

	return 0;
}


/*
 * Convert a property to a list of enums.  Return -1 if there was an error.
 */

static int convertEnumList(propDef *pd,propValidDef *pvd,propValueDef *pdd,PyObject *arg)
{
	int value, i;

	/*
	 * Enum lists are a special case in that the default value applies to
	 * the list, not one of it's elements.
	 */

	if (arg == NULL && propValueIsDefault(pdd))
		return pdd -> defenum;

	/* Check it is a valid list. */

	if (checkList(pd,pvd,arg) < 0)
		return -1;

	/* Convert each element. */

	value = 0;

	for (i = 0; i < PyList_GET_SIZE(arg); ++i)
	{
		int enumval;

		if (convertEnum(pd,pvd,pdd,PyList_GET_ITEM(arg,i),&enumval) < 0)
			return -1;

		if (value & enumval != 0)
		{
			char *name;
			enumValueDef *evd;

			/* Find the name of this value. */

			name = "Internal error";

			for (evd = pd -> ed -> values; evd -> name != NULL; ++evd)
				if (evd -> intval == enumval)
				{
					name = evd -> name;
					break;
				}

			PyErr_Format(PyExc_TypeError,"The %s property contains '%s' more than once",pd -> name,name);

			return -1;
		}

		value |= enumval;
	}

	/* Check the result is a valid combination. */

	if (pvd -> intlist != NULL)
	{
		int *il = pvd -> intlist;

		while (*il != value)
			if (*il++ == 0)
			{
				PyErr_Format(PyExc_TypeError,"The %s property has an invalid value in this context",pd -> name);

				return -1;
			}
	}

	return value;
}


/*
 * Check that an object is an instance of a VP class (or sub-class).
 */

static int isInstance(PyObject *arg,classDef *cd)
{
	return (PyInstance_Check(arg) && PyClass_IsSubclass((PyObject *)(((PyInstanceObject *)arg) -> in_class),cd -> classobj));
}


/*
 * Convert a property to an instance.  Return -1 if there was an error.
 */

static int convertInstance(propDef *pd,propValidDef *pvd,PyObject *arg,PyObject **p)
{
	if (arg == Py_None)
	{
		if (checkCanBeNone(pd,pvd))
			return -1;

		arg = NULL;
	}
	else if (arg != NULL && (arg = checkMakeInstance(pd -> cd,arg)) == NULL)
	{
		badType(pd);
		return -1;
	}

	*p = arg;

	return 0;
}


/*
 * Convert a property to a list of instances.  Return NULL if there was an
 * error.
 */

static PyObject *convertInstanceList(propDef *pd,propValidDef *pvd,PyObject *arg)
{
	int i;
	PyObject *new;

	/* Check it is a valid list. */

	if (checkList(pd,pvd,arg) < 0)
		return NULL;

	/* Copy the list while converting each element. */

	if ((new = PyList_New(PyList_GET_SIZE(arg))) == NULL)
		return NULL;

	for (i = 0; i < PyList_GET_SIZE(arg); ++i)
	{
		PyObject *itm = PyList_GET_ITEM(arg,i), *newitm;

		if (convertInstance(pd,pvd,itm,&newitm) < 0)
		{
			Py_DECREF(new);

			return NULL;
		}

		PyList_SET_ITEM(new,i,newitm);
	}

	return new;
}


/*
 * Check an object is an instance and return the wrapped object.  If it is not
 * an instance then try and create an instance with it.  If it fails the
 * exception is likely to be misleading, or at least confusing.
 */

static PyObject *checkMakeInstance(classDef *cd,PyObject *val)
{
	pymodPyBase *bw;
	PyObject *args;
 
	/* Just copy it if it is an instance. */

	if (isInstance(val,cd))
		Py_INCREF(val);
	else if ((args = PyTuple_New(1)) == NULL)
		return NULL;
	else
	{ 
		/* Try and create an instance using the value as a property. */
 
		PyTuple_SET_ITEM(args,0,val);
		Py_INCREF(val);
 
		val = PyInstance_New(cd -> classobj,args,NULL);
 
		Py_DECREF(args);

		if (val == NULL)
			return NULL;
	}

	/* Get the wrapped object from the instance. */

	if ((bw = pymodGetWrappedObject(val)) == NULL)
	{
		Py_DECREF(val);
		return NULL;
	}

	return (PyObject *)bw;
}
 

/*
 * Check that a property that should be a list really is one.  Return -1 if
 * there was an error.
 */

static int checkList(propDef *pd,propValidDef *pvd,PyObject *arg)
{
	if (arg == NULL || !PyList_Check(arg) || (PyList_GET_SIZE(arg) == 0 && !propValidListIsOptional(pvd)))
	{
		char *m1, *m2;

		propDesc(pd,&m1,&m2);

		PyErr_Format(PyExc_TypeError,"The %s property must be a %slist of %s%ss",pd -> name,(propValidListIsOptional(pvd) ? "" : "non-empty"),m1,m2);

		return -1;
	}

	return 0;
}


/*
 * Raise an exception about a bad type.
 */

static void badType(propDef *pd)
{
	char *m1, *m2;

	propDesc(pd,&m1,&m2);

	if (propIsList(pd))
		PyErr_Format(PyExc_TypeError,"Each element of the %s property must be a %s%s",pd -> name,m1,m2);
	else
		PyErr_Format(PyExc_TypeError,"The %s property must be a %s%s",pd -> name,m1,m2);
}


/*
 * Return the strings that describe the type of a property.
 */

static void propDesc(propDef *pd,char **desc1,char **desc2)
{
	char *m1, *m2;

	switch (pd -> type)
	{
	intProp:
		m1 = "integer";
		m2 = " value";
		break;

	uintProp:
		m1 = "unsigned integer";
		m2 = " value";
		break;

	doubleProp:
		m1 = "floating point";
		m2 = " value";
		break;

	case stringProp:
		m1 = "string";
		m2 = " value";
		break;

	case enumProp:
		m1 = pd -> ed -> type -> tp_name;
		m2 = " type";
		break;

	case instanceProp:
		m1 = pd -> cd -> name;
		m2 = " instance";
		break;

	case pyobjectProp:
		m1 = "Python object";
		m2 = "";
		break;
	}

	*desc1 = m1;
	*desc2 = m2;
}


/*
 * Delete a set of properties.
 */
 
void pymodPropsDelete(classPropDef *cpd,...)
{
	va_list va;

	va_start(va,cpd);

	while (cpd -> prop != NULL)
	{
		void *ptr = va_arg(va,void *);

		deleteProp(cpd -> prop,ptr);

		++cpd;
	}

	va_end(va);
}


/*
 * Delete a set of properties from a VP instance.
 */

void pymodPropsDeleteSelf(pymodPyBase *self)
{
	classPropDef *cpd;

	if ((cpd = self -> base.cd -> props) != NULL)
        	while (cpd -> prop != NULL)
        	{
                	void *ptr = (void *)self + cpd -> prop -> offset;

			deleteProp(cpd -> prop,ptr);

			++cpd;
		}
}
 

/*
 * Delete a single property.
 */

static void deleteProp(propDef *pd,void *ptr)
{
	if (pd -> dtor != NULL)
		(*pd -> dtor)(ptr);
	else if (pd -> type == instanceProp)
	{
		if (propIsList(pd))
		{
			PyObject *list = *(PyObject **)ptr;
 
			if (list != NULL)
			{
				int i;

				/*
				 * Replace the wrapped objects in the list
				 * (which don't have references) with the real
				 * instances (which do).
				 */

				for (i = 0; i < PyList_GET_SIZE(list); ++i)
				{
					PyObject *itm = PyList_GET_ITEM(list,i);

					if (itm == NULL)
					{
						Py_INCREF(Py_None);
						itm = Py_None;
					}
					else
						itm = ((pymodPyBase *)itm) -> base.instance;

					PyList_SET_ITEM(list,i,itm);
				}

				/* Now clean up all the references. */

				Py_DECREF(list);
				*(PyObject **)ptr = NULL;
			}
		}
		else
		{
			pymodPyBase *bw = *(pymodPyBase **)ptr;
 
			if (bw != NULL)
			{
				Py_DECREF(bw -> base.instance);
				*(pymodPyBase **)ptr = NULL;
			}
		}
	}
	else if (pd -> type == stringProp)
	{
		if (propIsList(pd))
		{
			PyObject *list = *(PyObject **)ptr;
 
			if (list != NULL)
			{
				int i;

				/* Replace any NULL elements with None. */

				for (i = 0; i < PyList_GET_SIZE(list); ++i)
					if (PyList_GET_ITEM(list,i) == NULL)
					{
						Py_INCREF(Py_None);
						PyList_SET_ITEM(list,i,Py_None);
					}

				/* Now clean up all the references. */

				Py_DECREF(list);
				*(PyObject **)ptr = NULL;
			}
		}
		else
		{
			PyObject *po = *(PyObject **)ptr;
 
			if (po != NULL)
			{
				Py_DECREF(po);
				*(PyObject **)ptr = NULL;
			}
		}
	}
	else if (pd -> type == pyobjectProp)
	{
		PyObject *po = *(PyObject **)ptr;
 
		if (po != NULL)
		{
			Py_DECREF(po);
			*(PyObject **)ptr = NULL;
		}
	}
}
