/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This header files defines the internal interfaces of the Python module
 * utility library.
 */


#ifndef _PYMODINT_H
#define	_PYMODINT_H

#include <sys/types.h>

#include "pymod.h"
#include "pylibmod.h"


/* Get the address of a toolkit pointer from a wrapper. */

#define	getOptTkPtr(wo)		((wo) == NULL ? NULL : (wo) -> base.tkPtr)


/* Get a string from a string property. */

#define	getOptString(po)	((po) == NULL ? NULL : PyString_AS_STRING(po))


/* Flag operations for a wrapper. */

#define	WRAP_DOING_SIGS		(1 << 0)	/* Currently raising signals. */

#define	wrapperIsDoingSigs(wo)		((wo) -> base.flags & WRAP_DOING_SIGS)
#define	wrapperSetDoingSigs(wo)		((wo) -> base.flags |= WRAP_DOING_SIGS)
#define	wrapperResetDoingSigs(wo)	((wo) -> base.flags &= ~WRAP_DOING_SIGS)


/* This defines an enum value. */

typedef struct {
	char		*name;			/* The value name. */
	int		intval;			/* The value as an integer. */
	PyObject	*objval;		/* The value as an object. */
} enumValueDef;


/* This defines an enum. */

typedef struct {
	PyTypeObject	*type;			/* The Python type. */
	enumValueDef	*values;		/* The enum values. */
} enumDef;


/* This defines the type of a property. */

typedef enum {
	stringProp,				/* A string. */
	enumProp,				/* An enum value. */
	doubleProp,				/* A double value. */
	intProp,				/* A signed integer value. */
	uintProp,				/* An unsigned integer value. */
	pyobjectProp,				/* Any Python object. */
	bespokeProp,				/* A bespoke property. */
	instanceProp				/* A class instance. */
} propType;


/* This defines valid values of a property. */

#define	PROP_VALID_OVERRIDE	(1 << 0)	/* Use this as the default. */
#define	PROP_VALID_EMPTY_OK	(1 << 1)	/* The list can be empty. */
#define	PROP_VALID_NONE_OK	(1 << 2)	/* The property can be None. */

#define	propValidOverride(pvd)		((pvd) -> flags & PROP_VALID_OVERRIDE)
#define	propValidListIsOptional(pvd)	((pvd) -> flags & PROP_VALID_EMPTY_OK)
#define	propValidCanBeNone(pvd)		((pvd) -> flags & PROP_VALID_NONE_OK)

typedef struct {
	int		flags;			/* The value flags. */
	int		*intlist;		/* Valid enum list values. */
} propValidDef;


/* This defines a default value of a property. */

#define	PROP_VALUE_OVERRIDE	(1 << 0)	/* Use this as the default. */
#define	PROP_VALUE_IS_DEFAULT	(1 << 1)	/* There is a default value. */
#define	PROP_VALUE_IS_NONE	(1 << 2)	/* The default is None. */

#define	propValueOverride(pdd)	((pdd) -> flags & PROP_VALUE_OVERRIDE)
#define	propValueIsDefault(pdd)	((pdd) -> flags & PROP_VALUE_IS_DEFAULT)
#define	propValueIsNone(pdd)	((pdd) -> flags & PROP_VALUE_IS_NONE)

typedef struct {
	int		flags;			/* The value flags. */
	int		defenum;		/* Default if an enum. */
	char		*defstr;		/* Default if a string. */
	long		defint;			/* Default if an integer. */
} propValueDef;


/* This defines a property. */

#define	PROP_FLAG_LIST		(1 << 0)	/* A list of types. */
#define	PROP_FLAG_NOT_CTOR	(1 << 1)	/* Can't be set in a ctor. */

#define	propIsList(pd)		((pd) -> flags & PROP_FLAG_LIST)
#define	propNotInCtor(pd)	((pd) -> flags & PROP_FLAG_NOT_CTOR)

typedef struct {
	char		*name;			/* The property name. */
	propType	type;			/* The property type. */
	size_t		offset;			/* The offset of the value. */
	int		flags;			/* The property flags. */
	propValidDef	defdefvalid;		/* The default default valid. */
	propValueDef	defdefvalue;		/* The default default value. */
	enumDef		*ed;			/* If it is an enum. */
	struct _classDef	*cd;		/* If it is an instance. */
	int		(*convFrom)(PyObject *,void *);	/* If it is bespoke. */
	PyObject	*(*convTo)(void *);	/* If it is bespoke. */
	void		(*dtor)(void *);	/* Optional property dtor. */
	PyObject	*nameobj;		/* The name as an object. */
} propDef;


/*
 * This defines a property as used by a particular class.
 */

typedef struct {
	propDef		*prop;			/* The property definition. */
	int		(*setprop)(pymodPyBase *);	/* The set property handler. */
	propValidDef	defvalid;		/* Overide the type default. */
	propValueDef	defvalue;		/* Overide the type default. */
} classPropDef;


/* This defines a connection. */

#define	CONN_IS_STALE		(1 << 0)	/* The connection is stale. */

#define	connIsStale(cd)		((cd) -> flags & CONN_IS_STALE)
#define	connSetStale(cd)	((cd) -> flags |= CONN_IS_STALE)

struct _connectionDef {
	int		flags;			/* Connection flags. */
	signalDef	*sd;			/* The signal. */
	PyObject	*slot;			/* The slot object. */
	connectionDef	*next;			/* The next in the list. */
};


/*
 * This is used as a place holder for the GUI dtor.  We normally overlay it
 * with the full table of GUI drivers.
 */

typedef struct {
	void		(*guidtor)(void *);	/* The GUI dtor. */
} guiDriversDef;


/* This defines a VP class. */

struct _classDef {
	char 		*name;			/* The class name. */
	classPropDef	*props;			/* The class properties. */
	PyMethodDef	*methods;		/* The class methods. */
	enumDef		**enums;		/* The enums. */
	signalDef	**signals;		/* The signals. */
	int		(*ctor)(pymodPyBase *,PyObject *,PyObject *);
	PyTypeObject	*pytype;		/* The type of the wrapper. */
	PyMethodDef	*funcs;			/* The class functions. */
	int		(*init)(classDef *);	/* The class init function. */
	PyObject	*classobj;		/* The Python class object. */
	guiDriversDef	*drivers;		/* Class drivers. */
};


/* This defines an enum object. */
 
typedef struct {
	PyObject_HEAD
	enumValueDef	*evd;			/* The raw value definition. */
	classDef	*cd;			/* The defining class. */
} enumObject;


/* This defines a signal object. */

typedef struct {
	PyObject_HEAD
	signalDef	*sd;			/* The real signal. */
	classDef	*cd;			/* The defining class. */
} signalObject;


extern PyTypeObject pymodSignalType;


/*****************************************************************************/
/* Declarations for the Color class.                                         */
/*****************************************************************************/

typedef struct _pymodPyColor pymodPyColor;

typedef struct {
	pymodDataBase		base;
	PyObject		*rgb;		/* The color as a string. */
	pymodColorDef		color;		/* The parsed color. */
} pymodDataColor;

struct _pymodPyColor {
	PyObject_HEAD
	pymodDataColor		color;
};

extern classDef pymodClassColor;

extern propDef pymodPropColorRGB;

#define	pymodDeallocColor	pymodDeallocBase


/*****************************************************************************/
/* Declarations for the Control class.                                       */
/*****************************************************************************/

typedef struct _pymodPyControl pymodPyControl;
typedef struct _pymodPyContainer pymodPyContainer;

typedef struct {
	pymodDataBase		base;
	pymodPyContainer	*parent;	/* The parent container. */
	long			column;		/* The grid column. */
	long			row;		/* The grid row. */
	pymodPyColor		*background;	/* The color of background. */
} pymodDataControl;

struct _pymodPyControl {
	PyObject_HEAD
	pymodDataControl	control;
};

extern classDef pymodClassControl;

extern propDef pymodPropControlParent;
extern propDef pymodPropControlColumn;
extern propDef pymodPropControlRow;

#define	pymodDeallocControl	pymodDeallocBase


/*****************************************************************************/
/* Declarations for the Container class.                                     */
/*****************************************************************************/

typedef struct {
	pymodDataControl	control;
	PyObject		*children;	/* List of children. */
} pymodDataContainer;

struct _pymodPyContainer {
	PyObject_HEAD
	pymodDataContainer	container;
};

extern classDef pymodClassContainer;

extern propDef pymodPropContainerChildren;

#define	pymodDeallocContainer	pymodDeallocControl


/*****************************************************************************/
/* Declarations for the ApplicationWindow class.                             */
/*****************************************************************************/

typedef struct _pymodPyApplicationWindow pymodPyApplicationWindow;

typedef struct {
	pymodDataContainer	container;
	PyObject		*argv;		/* Application arguments. */
	PyObject		*name;		/* Application defined name. */
	pylibAppWindow		*aw;		/* pylib reference. */
} pymodDataApplicationWindow;

struct _pymodPyApplicationWindow {
	PyObject_HEAD
	pymodDataApplicationWindow	applicationWindow;
};

extern classDef pymodClassApplicationWindow;

extern propDef pymodPropApplicationWindowArgv;
extern propDef pymodPropApplicationWindowName;

void pymodDeallocApplicationWindow(PyObject *);


/*****************************************************************************/
/* Declarations for the TopLevel class.                                      */
/*****************************************************************************/

typedef struct _pymodPyTopLevel pymodPyTopLevel;

typedef struct {
	pymodDataContainer	container;
	PyObject		*title;		/* The text of the title. */
} pymodDataTopLevel;

struct _pymodPyTopLevel {
	PyObject_HEAD
	pymodDataTopLevel	topLevel;
};

extern classDef pymodClassTopLevel;

extern propDef pymodPropTopLevelTitle;

#define	pymodDeallocTopLevel	pymodDeallocContainer


/*****************************************************************************/
/* Declarations for the Dialog class.                                        */
/*****************************************************************************/

typedef struct _pymodPyDialog pymodPyDialog;

typedef struct {
	pymodDataTopLevel	topLevel;
	int			buttons;	/* The dialog buttons. */
	int			closeBox;	/* The button emulated. */
	PyObject		*continueText;	/* The Continue button text. */
	int			defaultButton;	/* The default button. */
	int			modality;	/* The modality. */
	PyObject		*noText;	/* The No button text. */
	PyObject		*result;	/* The result object. */
	PyObject		*yesText;	/* The Yes button text. */
	struct pylibContext	*context;	/* While we are interacting. */
} pymodDataDialog;

struct _pymodPyDialog {
	PyObject_HEAD
	pymodDataDialog		dialog;
};

extern classDef pymodClassDialog;

extern enumDef pymodEnumDialogButton;
extern enumDef pymodEnumDialogModality;

extern propDef pymodPropDialogButtons;
extern propDef pymodPropDialogContinueText;
extern propDef pymodPropDialogModality;
extern propDef pymodPropDialogNoText;
extern propDef pymodPropDialogYesText;

#define	pymodDeallocDialog	pymodDeallocTopLevel


/*****************************************************************************/
/* Declarations for the Message class.                                       */
/*****************************************************************************/

typedef struct _pymodPyMessage pymodPyMessage;

typedef struct {
	pymodDataDialog		dialog;
	PyObject		*text;		/* The text of the message. */
} pymodDataMessage;

struct _pymodPyMessage {
	PyObject_HEAD
	pymodDataMessage	message;
};

extern classDef pymodClassMessage;

extern propDef pymodPropMessageText;

#define	pymodDeallocMessage	pymodDeallocDialog


/*****************************************************************************/
/* Declarations for the Grid class.                                          */
/*****************************************************************************/

typedef struct _pymodPyGrid pymodPyGrid;

typedef struct {
	pymodDataContainer	container;
} pymodDataGrid;

struct _pymodPyGrid {
	PyObject_HEAD
	pymodDataGrid		grid;
};

extern classDef pymodClassGrid;

#define	pymodDeallocGrid	pymodDeallocContainer


/*****************************************************************************/
/* Declarations for the HBox class.                                          */
/*****************************************************************************/

typedef struct _pymodPyHBox pymodPyHBox;

typedef struct {
	pymodDataContainer	container;
} pymodDataHBox;

struct _pymodPyHBox {
	PyObject_HEAD
	pymodDataHBox		hBox;
};

extern classDef pymodClassHBox;

#define	pymodDeallocHBox	pymodDeallocContainer


/*****************************************************************************/
/* Declarations for the VBox class.                                          */
/*****************************************************************************/

typedef struct _pymodPyVBox pymodPyVBox;

typedef struct {
	pymodDataContainer	container;
} pymodDataVBox;

struct _pymodPyVBox {
	PyObject_HEAD
	pymodDataVBox		vBox;
};

extern classDef pymodClassVBox;

#define	pymodDeallocVBox	pymodDeallocContainer


/*****************************************************************************/
/* Declarations for the Label class.                                         */
/*****************************************************************************/

typedef struct _pymodPyLabel pymodPyLabel;

typedef struct {
	pymodDataControl	control;
	PyObject		*text;		/* The text of the label. */
} pymodDataLabel;

struct _pymodPyLabel {
	PyObject_HEAD
	pymodDataLabel		label;
};

extern classDef pymodClassLabel;

extern propDef pymodPropLabelText;

#define	pymodDeallocLabel	pymodDeallocControl


/*****************************************************************************/
/* Declarations for the Chart class.                                         */
/*****************************************************************************/

typedef struct _pymodPyChart pymodPyChart;

typedef struct {
	pymodDataControl	control;
	long			height;		/* The chart height. */
	pymodPyColor		*labelColor;	/* The color of any labels. */
	PyObject		*title;		/* The text of the title. */
	pymodPyColor		*titleColor;	/* The color of the title. */
	long			width;		/* The chart width. */
} pymodDataChart;

struct _pymodPyChart {
	PyObject_HEAD
	pymodDataChart		chart;
};

extern classDef pymodClassChart;

extern propDef pymodPropChartHeight;
extern propDef pymodPropChartLabelColor;
extern propDef pymodPropChartTitle;
extern propDef pymodPropChartTitleColor;
extern propDef pymodPropChartWidth;

#define	pymodDeallocChart	pymodDeallocControl


/*****************************************************************************/
/* Declarations for the GraphPoint class.                                    */
/*****************************************************************************/

typedef struct _pymodPyGraphPoint pymodPyGraphPoint;

typedef struct {
	double			simple;		/* The simple value. */
	double			high;		/* The high value. */
	double			low;		/* The low value. */
	double			close;		/* The close value. */
} pymodBespokeGraphPointValue;

typedef struct {
	pymodDataBase		base;
	pymodBespokeGraphPointValue	value;	/* The value of the point. */
	pymodPyColor		*color;		/* The color of the point. */
} pymodDataGraphPoint;

struct _pymodPyGraphPoint {
	PyObject_HEAD
	pymodDataGraphPoint	graphPoint;
};

extern classDef pymodClassGraphPoint;

extern propDef pymodPropGraphPointColor;
extern propDef pymodPropGraphPointValue;

#define	pymodDeallocGraphPoint	pymodDeallocBase


/*****************************************************************************/
/* Declarations for the GraphSeries class.                                   */
/*****************************************************************************/

typedef struct _pymodPyGraphSeries pymodPyGraphSeries;

typedef struct {
	pymodDataBase		base;
	PyObject		*values;	/* The list of data points. */
	pymodPyColor		*color;		/* The color of the series. */
} pymodDataGraphSeries;

struct _pymodPyGraphSeries {
	PyObject_HEAD
	pymodDataGraphSeries	graphSeries;
};

extern classDef pymodClassGraphSeries;

extern propDef pymodPropGraphSeriesColor;
extern propDef pymodPropGraphSeriesValues;

#define	pymodDeallocGraphSeries	pymodDeallocBase


/*****************************************************************************/
/* Declarations for the Graph class.                                         */
/*****************************************************************************/

typedef struct _pymodPyGraph pymodPyGraph;

typedef struct {
	pymodDataChart		chart;
	pymodPyGraphSeries	*comboSeries;	/* The combo series. */
	PyObject		*comboTitle;	/* The optional combo title. */
	pymodPyColor		*comboTitleColor;	/* The combo title color. */
	int			graphType;	/* The graph's type. */
	PyObject		*labels;	/* The list of labels. */
	pymodPyColor		*plotColor;	/* The default plot color. */
	PyObject		*series;	/* The list of series. */
	PyObject		*xTitle;	/* The optional x title. */
	pymodPyColor		*xTitleColor;	/* The x title color. */
	PyObject		*yTitle;	/* The optional y title. */
	pymodPyColor		*yTitleColor;	/* The y title color. */
} pymodDataGraph;

struct _pymodPyGraph {
	PyObject_HEAD
	pymodDataGraph		graph;
};

extern classDef pymodClassGraph;

extern propDef pymodPropGraphComboSeries;
extern propDef pymodPropGraphComboTitle;
extern propDef pymodPropGraphComboTitleColor;
extern propDef pymodPropGraphGraphType;
extern propDef pymodPropGraphLabels;
extern propDef pymodPropGraphPlotColor;
extern propDef pymodPropGraphSeries;
extern propDef pymodPropGraphXTitle;
extern propDef pymodPropGraphXTitleColor;
extern propDef pymodPropGraphYTitle;
extern propDef pymodPropGraphYTitleColor;

#define	pymodDeallocGraph	pymodDeallocChart


/*****************************************************************************/
/* Declarations for the PieSlice class.                                      */
/*****************************************************************************/

typedef struct _pymodPyPieSlice pymodPyPieSlice;

typedef struct {
	pymodDataBase		base;
	PyObject		*label;		/* The optional label. */
	pymodPyColor		*color;		/* The color of the slice. */
	long			offset;		/* The offset of the slice. */
	double			value;		/* The value of the slice. */
} pymodDataPieSlice;

struct _pymodPyPieSlice {
	PyObject_HEAD
	pymodDataPieSlice	pieSlice;
};

extern classDef pymodClassPieSlice;

extern enumDef pymodEnumPieSliceSliceState;

extern propDef pymodPropPieSliceColor;
extern propDef pymodPropPieSliceLabel;
extern propDef pymodPropPieSliceOffset;
extern propDef pymodPropPieSliceValue;

#define	pymodDeallocPieSlice	pymodDeallocBase


/*****************************************************************************/
/* Declarations for the PieChart class.                                      */
/*****************************************************************************/

typedef struct _pymodPyPieChart pymodPyPieChart;

typedef struct {
	pymodDataChart		chart;
	pymodPyColor		*outlineColor;	/* The color of any outline. */
	PyObject		*series;	/* The list of slices. */
} pymodDataPieChart;

struct _pymodPyPieChart {
	PyObject_HEAD
	pymodDataPieChart	pieChart;
};

extern classDef pymodClassPieChart;

extern propDef pymodPropPieChartOutlineColor;
extern propDef pymodPropPieGraphSeries;

#define	pymodDeallocPieChart	pymodDeallocChart


/*****************************************************************************/


#define	pymodInitControl2	pymodContainerAddChild2

int pymodInitClasses(PyObject *);
int pymodParseArgs(classDef *,pymodPyBase *,PyObject *,PyObject *,classPropDef *,char *,...);
PyObject *pymodCallCtor(PyObject *,PyObject *);
PyObject *pymodCallGetattr(PyObject *,PyObject *);
PyObject *pymodCallSetattr(PyObject *,PyObject *);
PyObject *pymodConnect(PyObject *,PyObject *);
PyObject *pymodDisconnect(PyObject *,PyObject *);
PyObject *pymodEnumRepr(enumObject *);
PyObject *pymodConvertEnum(enumDef *,int);
pymodPyBase *pymodGetWrappedObject(PyObject *);
int pymodNullHandler(pymodPyBase *);
int pymodHandleSignal(pymodPyBase *,signalDef *,char *,...);
int pymodContainerAddChild1(pymodPyBase *);
void pymodContainerAddChild2(pymodPyBase *);
int pymodInitControl1(pymodPyBase *);
int pymodConvertDouble(PyObject *,double *);
void pymodPropsDelete(classPropDef *,...);
void pymodPropsDeleteSelf(pymodPyBase *);
void pymodDeallocBase(PyObject *);
void pymodBadToolkitCtor(char *);
unsigned long pymodChartColor(pymodPyColor *);
pymodPyColor *pymodColorNew(pymodColorDef *);

#endif
