/*
 * Copyright (c) 2000 Phil Thompson (phil@river-bank.demon.co.uk)
 *
 * This module implements the signal mechanism.
 */


#include <stdarg.h>

#include "pymodint.h"


static void applySignal(signalDef *,PyObject *,PyObject *);
static void removeConnection(connectionDef **);
static PyObject *signalRepr(signalObject *);
static pymodPyBase *getSignalWrapper(PyObject *,signalObject *);


PyTypeObject pymodSignalType = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,
	"Signal",
	sizeof (signalObject),
	0,
	0,
	0,
	0,
	0,
	0,
	(reprfunc)signalRepr,
};


/*
 * Implement connect().
 */

PyObject *pymodConnect(PyObject *ignore,PyObject *args)
{
	pymodPyBase *wo;
	PyObject *inst, *slot;
	signalObject *so;
	connectionDef *cd;

	/* Get the instance name, the signal and the slot. */

	if (!PyArg_ParseTuple(args,"O!O!O:connect",&PyInstance_Type,&inst,&pymodSignalType,&so,&slot))
		return NULL;

	if (!PyCallable_Check(slot))
	{
		PyErr_Format(PyExc_TypeError,"The slot must be a method or a function");
		return NULL;
	}

	/* Get the wrapped object and verify it can raise the signal. */

	if ((wo = getSignalWrapper(inst,so)) == NULL)
		return NULL;

	/*
	 * Create the new connection.  It must be put at the head of the list
	 * in case this is being done in a slot.  This ensures the connection
	 * wont be in effect until the next time this signal is raised.
	 */

	if ((cd = (connectionDef *)Py_Malloc(sizeof (connectionDef))) == NULL)
		return NULL;

	cd -> flags = 0;
	cd -> sd = so -> sd;
	cd -> slot = slot;
	cd -> next = wo -> base.connections;
	wo -> base.connections = cd;

	Py_INCREF(slot);

	Py_INCREF(Py_None);
	return Py_None;
}


/*
 * Implement disconnect().
 */

PyObject *pymodDisconnect(PyObject *ignore,PyObject *args)
{
	pymodPyBase *wo;
	PyObject *inst, *slot;
	signalObject *so;
	connectionDef *cd, **cdp;

	/* Get the instance name, the signal and the slot. */

	so = NULL;
	slot = NULL;

	if (!PyArg_ParseTuple(args,"O!|O!O:disconnect",&PyInstance_Type,&inst,&pymodSignalType,&so,&slot))
		return NULL;

	if (slot != NULL && !PyCallable_Check(slot))
	{
		PyErr_Format(PyExc_TypeError,"The slot must be a method or a function");
		return NULL;
	}

	/* Get the wrapped object and verify it can raise the signal. */

	if ((wo = getSignalWrapper(inst,so)) == NULL)
		return NULL;

	/* Go through each connection to see if it should be disconnected. */

	cdp = &wo -> base.connections;

	while ((cd = *cdp) != NULL)
		if ((so == NULL || cd -> sd == so -> sd) && (slot == NULL || cd -> slot == slot))
		{
			/*
			 * If we are in a slot then delay the removal of the
			 * link in the list.
			 */

			if (wrapperIsDoingSigs(wo))
			{
				connSetStale(cd);
				cdp = &cd -> next;
			}
			else
				removeConnection(cdp);
		}
		else
			cdp = &cd -> next;

	Py_INCREF(Py_None);
	return Py_None;
}


/*
 * Handle a signal, either by calling a virtual Python method if there is one,
 * or emitting the signal to any connected slots.  Returns non-zero if the
 * default action should be performed.
 */

int pymodHandleSignal(pymodPyBase *wo,signalDef *sd,char *fmt,...)
{
	int dodefault;
	PyObject *args, *methobj, *nargs;
	connectionDef *cd, **cdp;
	va_list va;

	PyErr_Clear();
	dodefault = 1;

	/* Create the argument tuple. */

	va_start(va,fmt);
	args = Py_VaBuildValue(fmt,va);
	va_end(va);

	if (args == NULL)
		goto unlock;

	/* Sanity check. */

	if (!PyTuple_Check(args))
	{
		PyErr_Format(PyExc_SystemError,"Internal error: signal arguments not a tuple");
		goto unlock;
	}

	/* See if there is a virtual method. */

	if ((methobj = PyObject_GetAttr(wo -> base.instance,sd -> nameobj)) != NULL && !PyMethod_Check(methobj))
	{
		Py_DECREF(methobj);
		methobj = NULL;
	}

	if (methobj != NULL)
	{
		applySignal(sd,methobj,args);
		Py_DECREF(methobj);
		dodefault = 0;
	}

	/* Now handle any explicit connections. */

	wrapperSetDoingSigs(wo);

	nargs = NULL;

	for (cd = wo -> base.connections; cd != NULL; cd = cd -> next)
	{
		if (cd -> sd != sd || connIsStale(cd))
			continue;

		if (nargs == NULL)
		{
			int i;

			/*
			 * Prepend the instance raising the signal to the
			 * arguments. We can't use _PyTuple_Resize() because it
			 * insists that the reference count of the tuple is 1.
			 * Because Python can cache small tuples this might not
			 * be the case.
			 */

			if ((nargs = PyTuple_New(PyTuple_GET_SIZE(args) + 1)) == NULL)
				break;

			PyTuple_SET_ITEM(nargs,0,wo -> base.instance);
			Py_INCREF(wo -> base.instance);

			for (i = 0; i < PyTuple_GET_SIZE(args); ++i)
			{
				PyObject *obj = PyTuple_GET_ITEM(args,i);

				Py_INCREF(obj);
				PyTuple_SET_ITEM(nargs,i + 1,obj);
			}

			Py_DECREF(args);
			args = nargs;
		}

		applySignal(sd,cd -> slot,args);
		dodefault = 0;
	}

	wrapperResetDoingSigs(wo);

	/* Remove any stale connections. */

	cdp = &wo -> base.connections;

	while ((cd = *cdp) != NULL)
		if (connIsStale(cd))
			removeConnection(cdp);
		else
			cdp = &cd -> next;

	/* We've finished with the arguments. */

	Py_DECREF(args);

unlock:
	if (PyErr_Occurred() != NULL)
	{
		pylibException();
		dodefault = 0;
	}

	return dodefault;
}


/*
 * Apply a signal to a slot.
 */

static void applySignal(signalDef *sd,PyObject *meth,PyObject *args)
{
	PyObject *res;

	if ((res = PyEval_CallObject(meth,args)) == NULL)
		pylibException();
	else
	{
		if (res != Py_None)
		{
			PyErr_Format(PyExc_TypeError,"%s should return None",sd -> name);
			pylibException();
		}

		Py_DECREF(res);
	}
}


/*
 * The repr() function for the signal type.
 */
 
static PyObject *signalRepr(signalObject *so)
{
	char buf[100];
 
	sprintf(buf,"vp.%s.%s",so -> cd -> name,so -> sd -> name);
 
	return PyString_FromString(buf);
}


/*
 * Get the wrapped object from an instance and check it can raise the signal.
 */

static pymodPyBase *getSignalWrapper(PyObject *inst,signalObject *so)
{
	int valid;
	pymodPyBase *wo;

	if ((wo = pymodGetWrappedObject(inst)) == NULL)
	{
		PyErr_Format(PyExc_TypeError,"The %s class does not raise signals",PyString_AS_STRING(((PyInstanceObject *)inst) -> in_class -> cl_name));

		return NULL;
	}

	/* Check the signal is valid for the instance. */

	valid = 0;

	if (so == NULL)
		valid = 1;
	else if (wo -> base.cd -> signals != NULL)
	{
		signalDef **sdp;

		for (sdp = wo -> base.cd -> signals; *sdp != NULL; ++sdp)
			if (*sdp == so -> sd)
			{
				valid = 1;
				break;
			}
	}

	if (!valid)
	{
		PyErr_Format(PyExc_TypeError,"The %s class does not raise the %s signal",PyString_AS_STRING(((PyInstanceObject *)inst) -> in_class -> cl_name),so -> sd -> name);

		return NULL;
	}

	return wo;
}


/*
 * Remove a connection.
 */

static void removeConnection(connectionDef **cdp)
{
	connectionDef *cd = *cdp;

	*cdp = cd -> next;
	Py_DECREF(cd -> slot);
	Py_Free((ANY *)cd);
}
